/* Analysis.do implements the statistical estimation for the DK data */


* log just for main estimates
log using RiskAndTime_Main.log, replace text name(mainlog)

* set characters
local Dalpha=char(225)
local Dbeta=char(226)
local Ddelta=char(228)
local Dlambda=char(235)
local Dgamma=char(227)
local Dmu=char(236)
local Dpi=char(240)

* storage for predicted choices
generate Pchoices=.

* save data
egen obs=seq()
sort obs
save tmp, replace

* specify default level of background consumption
global omega=118

* specify default lambda values for each task, where 0 refers to the early payment option (integer values only)
global lambda0=$DEFAULTlambda
global lambda1=$DEFAULTlambda
global lambda2=$DEFAULTlambda
global lambda3=$DEFAULTlambda
global lambda4=$DEFAULTlambda
global lambda5=$DEFAULTlambda
global lambda6=$DEFAULTlambda



** RISK AVERSION CHOICES ONLY **
* risk aversion only with no background wealth
constrain define 1 [LNw]_cons = -5
ml model lf ML_crra (r: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 = ) (LNw: ) (mu: ) if ra==1 , cluster(id) missing maximize technique(dfp) difficult constraint(1)
ml display

* risk aversion only with background wealth set to 118 kroner
constrain define 1 [LNw]_cons = 4.771
ml model lf ML_crra (r: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 = ) (LNw: ) (mu: ) if ra==1 , cluster(id) missing maximize technique(dfp) difficult constraint(1)
ml display

* allow for demographics
ml model lf ML_crra (r: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 = $demog ) (LNw: ) (mu: ) if ra==1 , cluster(id) missing maximize technique(dfp) difficult continue constraint(1)
ml display
predictnl p_ra=xb(r)
keep id p_ra
sort id
save tmpRA, replace





** RISK NEUTRALITY **
* return to data
use tmp, clear

* Table 3, Panel B: Exponential discounting with risk neutrality
ml model lf MLex_rn_lam (LNdelta: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (muIDR: ) if ra==0 , cluster(id) missing maximize technique(dfp) difficult init(-1.379 0.133, copy)
ml display
nlcom (delta: exp([LNdelta]_b[_cons]))


* Figure 2, Panel A: Demographics for RN case
use tmp, clear
ml model lf MLex_rn_lam (LNdelta: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (muIDR: ) if ra==0 , cluster(id) missing maximize technique(dfp) difficult init(-1.379 0.133, copy)

ml model lf MLex_rn_lam (LNdelta: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = $demog ) (muIDR: ) if ra==0 , cluster(id) missing maximize technique(dfp) difficult continue
ml display
nlcom (delta: exp([LNdelta]_b[_cons]))
predictnl p_deltaRN=exp(xb(LNdelta))
keep obs id p_deltaRN
sort obs
save tmp_fig2_panelA_RN, replace

* correlate RN discount rates with RA
sort id
merge id using tmpRA
bysort id: egen task=seq()
ktau p_deltaRN p_ra if task==1
erase tmpRA.dta



** JOINT ESTIMATION **
* return to data
use tmp, clear

* specify level of background consumption
global omega=118

* specify lambda values for each task, where 0 refers to the early payment option (integer values only)
global lambda0=$DEFAULTlambda
global lambda1=$DEFAULTlambda
global lambda2=$DEFAULTlambda
global lambda3=$DEFAULTlambda
global lambda4=$DEFAULTlambda
global lambda5=$DEFAULTlambda
global lambda6=$DEFAULTlambda


* Table 3, Panel A: Exponential discounting with CRRA
    ml model lf MLex_ra_crra_lam (r: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (LNdelta: ) (muRA: ) (muIDR: ), cluster(id) missing maximize technique(dfp) difficult init(0.746 -2.002 0.084 0.017, copy)
    ml display
    save tmpP, replace
    nlcom (delta: exp([LNdelta]_b[_cons]))
    predictnl p_delta=exp(xb(LNdelta))
    summ Pchoices choices if Pchoices~=. & choices~=-1

    * Test discount rate against RN estimate
    test [LNdelta]_cons   = -1.379448


* Look at correlation matrix estimates
    estat vce, corr

* get part of Figure 4
    GetEx
    twoway (rarea ExDisLO ExDisHI Horizon if Horizon>$HorizonVIEW & Horizon<=3, sort blcolor(gs14) bfcolor(gs14)) (connected ExDis Horizon if Horizon>$HorizonVIEW & Horizon<=3, msize(zero) sort), title("Exponential ", position(12) ring(0) size(large)) legend(off) ytitle("Discount Rate", size(medium)) xtitle("", size(vlarge)) xlabel(0 (1) 3, nogrid angle(horizontal)) ylabel(0(0.05)0.15 , nogrid angle(horizontal)) legend(off) saving(ExDisRA, replace)

* estimate using delta instead of LNdelta, so we get correlations
    use tmp, clear
    ml model lf MLex_ra_crra_lam_ (r: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (delta: ) (muRA: ) (muIDR: ), cluster(id) missing maximize technique(dfp) difficult init(0.746 0.14 0.084 0.017, copy)
    ml display
    estat vce, corr



* return to data
    use tmp, clear


* Figure 2, Panel A: Demographics for CRRA case
    ml model lf MLex_ra_crra_lam (r: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (LNdelta: ) (muRA: ) (muIDR: ), cluster(id) missing maximize technique(dfp) difficult init(0.746 -2.002 0.084 0.017, copy)
    ml display

    ml model lf MLex_ra_crra_lam (r: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = $demog ) (LNdelta: $demog ) (muRA: ) (muIDR: ), cluster(id) missing maximize technique(dfp) difficult continue
    ml display
    save tmpPP, replace

    predictnl p_deltaRA=exp(xb(LNdelta))
    predictnl p_raRA=xb(r)
    bysort id: egen task=seq()
    ktau p_deltaRA p_raRA if task==1
    drop task


* Test discount rates against RN estimates separately
test [LNdelta]_cons   = -1.480062
test [LNdelta]female  =  0.068578
test [LNdelta]young   = -0.041910
test [LNdelta]middle  =  0.149832
test [LNdelta]old     =  0.297648
test [LNdelta]single  =  0.122897
test [LNdelta]kids    =  0.202239
test [LNdelta]owner   =  0.012961
test [LNdelta]retired = -0.144949
test [LNdelta]student = -0.132009
test [LNdelta]skilled = -0.174563
test [LNdelta]longedu = -0.235804
test [LNdelta]IncLow  =  0.048170
test [LNdelta]IncHigh = -0.268628
test [LNdelta]copen   =  0.103358
test [LNdelta]city    =  0.120586

* Test discount rates against RN estimates jointly
test [LNdelta]_cons   = -1.480062
test [LNdelta]female  =  0.068578, accumulate
test [LNdelta]young   = -0.041910, accumulate
test [LNdelta]middle  =  0.149832, accumulate
test [LNdelta]old     =  0.297648, accumulate
test [LNdelta]single  =  0.122897, accumulate
test [LNdelta]kids    =  0.202239, accumulate
test [LNdelta]owner   =  0.012961, accumulate
test [LNdelta]retired = -0.144949, accumulate
test [LNdelta]student = -0.132009, accumulate
test [LNdelta]skilled = -0.174563, accumulate
test [LNdelta]longedu = -0.235804, accumulate
test [LNdelta]IncLow  =  0.048170, accumulate
test [LNdelta]IncHigh = -0.268628, accumulate
test [LNdelta]copen   =  0.103358, accumulate
test [LNdelta]city    =  0.120586, accumulate

* Figure 2, Panel A
    sort obs
    merge obs using tmp_fig2_panelA_RN
    tab _merge
    save tmp_fig2, replace
    bysort id: egen task=seq()
    summ p_deltaRN p_deltaRA if task==1
    twoway (kdensity p_deltaRN if task==1, lpattern(dash) lwidth(thick)) (kdensity p_deltaRA if task==1, lpattern(solid) lwidth(thick)), ytitle(Density, size(medlarge)) ylabel(, noticks nolabels) xtitle(Estimated Discount Rate, size(medlarge)) title("A. Allowing for" "Observable Heterogenity", position(2) ring(0) justification(center) box) legend(cols(1) order(1 "Assuming RN" 2 "Allowing RA")) legend(position(3) ring(0)) saving(fig2a, replace)

* Generate predicted choice pictures in the appendix
do predictC

* return to data
use tmp, clear

* trace out effects of varying omega for lambda=5
global omega=118
forvalues H=0(1)6 {
        global lambda`H'=5
}
foreach omegaX in 5 50 100 150 200 {
    global omega=`omegaX'
    di "Evaluating uniform lambda of 5 and omega of `omegaX':"
    ml model lf MLex_ra_crra_lam (r: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (LNdelta: ) (muRA: ) (muIDR: ), cluster(id) missing maximize technique(dfp) difficult init(0.746 -2.002 0.084 0.017, copy)
    ml display
    nlcom (delta: exp([LNdelta]_b[_cons]))
    predictnl delta_`omegaX' = exp(xb(LNdelta)), ci(delta_lo_`omegaX' delta_hi_`omegaX')
    predictnl r_`omegaX' = xb(r), ci(r_lo_`omegaX' r_hi_`omegaX')
}

* collapse estimates for each baseline consumption w and draw picture
collapse (mean) r_* delta_*
gen id=1
reshape long r_ r_lo_ r_hi_ delta_ delta_lo_ delta_hi_, i(id) j(w)
list, noobs
global r "0.5(0.1)1.0"
global d "0.05(0.025)0.2"
twoway (rarea r_hi_ r_lo_         w, sort lcolor(white) fcolor(gs14)) (connected r_ w,     lwidth(medthick) msize(zero)), ytitle("", orientation(horizontal)) ylabel($r, angle(horizontal)) xtitle("Baseline Consumption (`Domega')", size(large)) xlabel(0(50)200) title("Risk Aversion (r)",               position(2) ring(0) justification(center) box) legend(off) saving(tmp1, replace)
twoway (rarea delta_hi_ delta_lo_ w, sort lcolor(white) fcolor(gs14)) (connected delta_ w, lwidth(medthick) msize(zero)), ytitle("", orientation(horizontal)) ylabel($d, angle(horizontal)) xtitle("Baseline Consumption (`Domega')", size(large)) xlabel(0(50)200) title("Discount Rate (`Ddelta')", position(2) ring(0) justification(center) box) legend(off) saving(tmp2, replace)
graph combine tmp1.gph tmp2.gph, row(1) imargin(tiny) title("Figure 1: Estimated Discount Rates and Days of Consumption") saving(Figure1w5, replace)
erase tmp1.gph
erase tmp2.gph


* trace out effects of varying omega for lambda=1
use tmp, clear
global omega=118
forvalues H=0(1)6 {
        global lambda`H'=1
}
foreach omegaX in 5 50 100 150 200 {
    global omega=`omegaX'
    di "Evaluating uniform lambda of 1 and omega of `omegaX':"
    ml model lf MLex_ra_crra_lam (r: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (LNdelta: ) (muRA: ) (muIDR: ), cluster(id) missing maximize technique(dfp) difficult init(0.746 -2.002 0.084 0.017, copy)
    ml display
    nlcom (delta: exp([LNdelta]_b[_cons]))
    predictnl delta_`omegaX' = exp(xb(LNdelta)), ci(delta_lo_`omegaX' delta_hi_`omegaX')
    predictnl r_`omegaX' = xb(r), ci(r_lo_`omegaX' r_hi_`omegaX')
}

* collapse estimates for each baseline consumption w and draw picture
collapse (mean) r_* delta_*
gen id=1
reshape long r_ r_lo_ r_hi_ delta_ delta_lo_ delta_hi_, i(id) j(w)
list, noobs
global r "0.5(0.1)1.0"
global d "0.05(0.025)0.2"
twoway (rarea r_hi_ r_lo_         w, sort lcolor(white) fcolor(gs14)) (connected r_ w,     lwidth(medthick) msize(zero)), ytitle("", orientation(horizontal)) ylabel($r, angle(horizontal)) xtitle("Baseline Consumption (`Domega')", size(large)) xlabel(0(50)200) title("Risk Aversion (r)",               position(2) ring(0) justification(center) box) legend(off) saving(tmp1, replace)
twoway (rarea delta_hi_ delta_lo_ w, sort lcolor(white) fcolor(gs14)) (connected delta_ w, lwidth(medthick) msize(zero)), ytitle("", orientation(horizontal)) ylabel($d, angle(horizontal)) xtitle("Baseline Consumption (`Domega')", size(large)) xlabel(0(50)200) title("Discount Rate (`Ddelta')", position(2) ring(0) justification(center) box) legend(off) saving(tmp2, replace)
graph combine tmp1.gph tmp2.gph, row(1) imargin(tiny) title("Figure 1: Estimated Discount Rates and Days of Consumption") saving(Figure1w1, replace)
erase tmp1.gph
erase tmp2.gph


* trace out effects of varying omega for lambda=1 and alternate CRRA function
use tmp, clear
global omega=118
forvalues H=0(1)6 {
        global lambda`H'=1
}
foreach omegaX in 5 50 100 150 200 250 300 350 400 450 500 550 600 650 700 {
    global omega=`omegaX'
    di "Evaluating uniform lambda of 1 and omega of `omegaX':"
    ml model lf MLex_ra_crra2_lam (r: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (LNdelta: ) (muRA: ) (muIDR: ), cluster(id) missing maximize technique(dfp) difficult init(0.746 -2.002 0.084 0.017, copy)
    ml display
    nlcom (delta: exp([LNdelta]_b[_cons]))
    predictnl delta_`omegaX' = exp(xb(LNdelta)), ci(delta_lo_`omegaX' delta_hi_`omegaX')
    predictnl r_`omegaX' = xb(r), ci(r_lo_`omegaX' r_hi_`omegaX')
}

* collapse estimates for each baseline consumption w and draw picture
collapse (mean) r_* delta_*
gen id=1
reshape long r_ r_lo_ r_hi_ delta_ delta_lo_ delta_hi_, i(id) j(w)
list, noobs
global r "0.5(0.1)1.0"
global d "0.05(0.025)0.2"
twoway (rarea r_hi_ r_lo_         w, sort lcolor(white) fcolor(gs14)) (connected r_ w,     lwidth(medthick) msize(zero)), ytitle("", orientation(horizontal)) ylabel($r, angle(horizontal)) xtitle("Baseline Consumption (`Domega')", size(large)) xlabel(0(100)700) title("Risk Aversion (r)",               position(2) ring(0) justification(center) box) legend(off) saving(tmp1, replace)
twoway (rarea delta_hi_ delta_lo_ w, sort lcolor(white) fcolor(gs14)) (connected delta_ w, lwidth(medthick) msize(zero)), ytitle("", orientation(horizontal)) ylabel($d, angle(horizontal)) xtitle("Baseline Consumption (`Domega')", size(large)) xlabel(0(100)700) title("Discount Rate (`Ddelta')", position(2) ring(0) justification(center) box) legend(off) saving(tmp2, replace)
graph combine tmp1.gph tmp2.gph, row(1) imargin(tiny) title("Figure 1: Estimated Discount Rates and Days of Consumption") saving(Figure1w1, replace)
erase tmp1.gph
erase tmp2.gph


* return to data
use tmp, clear

* Figure 1: Trace out effects of increasing lambda
global omega=118
forvalues lambdaX=1(1)7 {
    forvalues H=0(1)6 {
        global lambda`H'=`lambdaX'
    }
    di "Evaluating uniform lambda of `lambdaX':"
    ml model lf MLex_ra_crra_lam (r: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (LNdelta: ) (muRA: ) (muIDR: ), cluster(id) missing maximize technique(dfp) difficult init(0.746 -2.002 0.084 0.017, copy)
    ml display
    nlcom (delta: exp([LNdelta]_b[_cons]))
    predictnl delta_`lambdaX' = exp(xb(LNdelta)), ci(delta_lo_`lambdaX' delta_hi_`lambdaX')
    predictnl r_`lambdaX' = xb(r), ci(r_lo_`lambdaX' r_hi_`lambdaX')
}

* collapse estimates for each day and draw picture
collapse (mean) r_* delta_*
gen id=1
reshape long r_ r_lo_ r_hi_ delta_ delta_lo_ delta_hi_, i(id) j(days)
list, noobs
global r "0.5(0.1)1.0"
global d "0.05(0.025)0.2"
twoway (rarea r_hi_ r_lo_         days, sort lcolor(white) fcolor(gs14)) (connected r_ days,     lwidth(medthick) msize(zero)), ytitle("", orientation(horizontal)) ylabel($r, angle(horizontal)) xtitle(Days, size(large)) xlabel(1(1)7) title("Risk Aversion (r)",               position(2) ring(0) justification(center) box) legend(off) saving(tmp1, replace)
twoway (rarea delta_hi_ delta_lo_ days, sort lcolor(white) fcolor(gs14)) (connected delta_ days, lwidth(medthick) msize(zero)), ytitle("", orientation(horizontal)) ylabel($d, angle(horizontal)) xtitle(Days, size(large)) xlabel(1(1)7) title("Discount Rate (`Ddelta')", position(2) ring(0) justification(center) box) legend(off) saving(tmp2, replace)
graph combine tmp1.gph tmp2.gph, row(1) imargin(tiny) title("Figure 1: Estimated Discount Rates and Days of Consumption") saving(Figure1, replace)
erase tmp1.gph
erase tmp2.gph


* return to data
use tmp, clear
global omega=118

* specify lambda values for each task, where 0 refers to the early payment option (integer values only)
global lambda0=$DEFAULTlambda
global lambda1=$DEFAULTlambda
global lambda2=$DEFAULTlambda
global lambda3=$DEFAULTlambda
global lambda4=$DEFAULTlambda
global lambda5=$DEFAULTlambda
global lambda6=$DEFAULTlambda

* Table 4, Panel B: Mazur-hyperbolic discounting with risk neutrality
ml model lf MLh_rn_lam (LNalphaD: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (muIDR: ) if ra==0 , cluster(id) missing maximize technique(dfp) difficult init(-1.379 0.133, copy)
ml display
nlcom (alpha: exp([LNalphaD]_b[_cons]))
GetHY
twoway (rarea HYDisLO HYDisHI Horizon if Horizon>$HorizonVIEW & Horizon<=3, sort blcolor(gs14) bfcolor(gs14)) (connected HYDis Horizon if Horizon>$HorizonVIEW & Horizon<=3, msize(zero) sort), title(Assuming Risk Neutrality, position(12) ring(0) size(large)) legend(off) ytitle("Discount Rate", size(medium)) xtitle("", size(vlarge)) ylabel(0(0.05)0.3 , nogrid angle(horizontal))  legend(off) saving(HYDisRN, replace)

* return to data
use tmp, clear

* Table 4, Panel A: Mazur-hyperbolic discounting with CRRA
ml model lf MLh_crra_lam (r: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (LNalphaD: ) (muRA: ) (muIDR: ), cluster(id) missing maximize technique(bfgs) difficult init(0.755 -1.968 0.082 0.017, copy)
ml display
nlcom (alphaD: exp([LNalphaD]_b[_cons]))

* Test discount rate against RN estimate
test [LNalphaD]_cons   = -1.309664


GetHY
twoway (rarea HYDisLO HYDisHI Horizon if Horizon>$HorizonVIEW & Horizon<=3, sort blcolor(gs14) bfcolor(gs14)) (connected HYDis Horizon if Horizon>$HorizonVIEW & Horizon<=3, msize(zero) sort), title(Allowing Risk Aversion, position(12) ring(0) size(large) ) legend(off) ytitle("", size(medium) orientation(vertical)) xtitle("", size(vlarge)) ylabel(0(0.05)0.3, nogrid angle(horizontal))  legend(off) saving(HYDisRA, replace)
twoway (rarea HYDisLO HYDisHI Horizon if Horizon>$HorizonVIEW & Horizon<=3, sort blcolor(gs14) bfcolor(gs14)) (connected HYDis Horizon if Horizon>$HorizonVIEW & Horizon<=3, msize(zero) sort), title(Hyperbolic, position(12) ring(0) size(large) ) legend(off) ytitle("", size(medium) orientation(vertical)) xtitle("", size(vlarge)) ylabel(0(0.05)0.15, nogrid angle(horizontal))  legend(off) saving(HYDisRA_, replace)

* Figure 3
graph combine HYDisRN.gph HYDisRA.gph, ycommon xcommon imargin(vsmall) col(3) title("Figure 3: Assuming Hyperbolic Discounting", size(vlarge)) b1title("Horizon in Years", position(6) ring(0)) saving(Figure3, replace)

* Figure 4
graph combine EXDisRA.gph HYDisRA_.gph, ycommon xcommon imargin(vsmall) col(3) title("Figure 4:" "Comparing Exponential and Hyperbolic Discount Rates", size(vlarge)) subtitle("Assuming concave utility functions and EUT") b1title("Horizon in Years", position(6) ring(0)) saving(Figure4, replace)

* return to data
use tmp, clear

* Prelec-hyperbolic discounting with risk neutrality
ml model lf MLp_rn_lam (betaD: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (alphaD_: ) (muIDR: ) if ra==0 , cluster(id) missing maximize technique(dfp) difficult init(0.135 1 0.133, copy)
ml display

* Prelec-hyperbolic with CRRA
ml model lf MLp_crra_lam (r: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (betaD: ) (alphaD_: ) (muRA: ) (muIDR: ), cluster(id) missing maximize technique(bfgs) difficult init(0.746 0.135 1 0.084 0.017, copy)
ml display





** MIXTURE MODELS **

* return to data
use tmp, clear

* specify level of background consumption
global omega=118

* specify lambda values for each task, where 0 refers to the early payment option (integer values only)
global lambda0=$DEFAULTlambda
global lambda1=$DEFAULTlambda
global lambda2=$DEFAULTlambda
global lambda3=$DEFAULTlambda
global lambda4=$DEFAULTlambda
global lambda5=$DEFAULTlambda
global lambda6=$DEFAULTlambda


* Table 5, Panel B: Mixture model of exponential and Mazur-hyperbolic discounting with risk neutrality
ml model lf MLmix_ex_h_rn_lam (LNdelta: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (LNalphaD: ) (kappa: ) (muIDR: ) if ra==0, cluster(id) missing maximize technique(dfp) difficult init(-1.455 0.433 -2.530 0.121, copy)
ml display
nlcom (delta: exp([LNdelta]_b[_cons])) (alpha: exp([LNalphaD]_b[_cons]))
nlcom (probEXP: 1/(1+exp([kappa]_b[_cons]))) (probHYP: exp([kappa]_b[_cons])/(1+exp([kappa]_b[_cons])))
nlcom (probSAME: 1/(1+exp([kappa]_b[_cons])) - exp([kappa]_b[_cons])/(1+exp([kappa]_b[_cons])))

* save data
save tmpMIXrn, replace

* get RN exponential part of Figure 4
GetEx
twoway (rarea ExDisLO ExDisHI Horizon if Horizon>$HorizonVIEW & Horizon<=3, sort blcolor(gs14) bfcolor(gs14)) (connected ExDis Horizon if Horizon>$HorizonVIEW & Horizon<=3, msize(zero) sort lwidth(medthick)), title("B. Assuming Risk Neutrality:" "Exponential Discounting (93%)", position(2) ring(0) justification(center) box) legend(off) ytitle("Discount Rate", size(medium)) xtitle("", size(vlarge)) xlabel(0(1)3, nogrid angle(horizontal)) ylabel(0(0.25)1.75 , nogrid angle(horizontal)) legend(off) saving(RNexp, replace)

* get RN hyperbolic part of Figure 4
use tmpMIXrn, clear
GetHY
twoway (rarea HYDisLO HYDisHI Horizon if Horizon>$HorizonVIEW & Horizon<=3, sort blcolor(gs14) bfcolor(gs14)) (connected HYDis Horizon if Horizon>$HorizonVIEW & Horizon<=3, msize(zero) sort lwidth(medthick)), title("B. Assuming Risk Neutrality:" "Hyperbolic Discounting (7%)", position(2) ring(0) justification(center) box) legend(off) ytitle("", size(medium) orientation(vertical)) xtitle("", size(vlarge)) ylabel(0(0.25)1.75, nogrid angle(horizontal))  legend(off) saving(RNhyp, replace)

* Figure 4 -- the RN bit
graph combine RNexp.gph RNhyp.gph, ycommon xcommon imargin(vsmall) col(2) b1title("Horizon in Years", position(6) ring(0)) saving(Figure4rn, replace)


* Table 5, Panel A: Mixture model of exponential and Mazur-hyperbolic discounting with CRRA
use tmp, clear
ml model lf MLmix_ex_h_lam (r: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (LNdelta: ) (LNalphaD: ) (kappa: ) (muRA: ) (muIDR: ), cluster(id) missing maximize technique(dfp) difficult init(0.7709 -2.3318 -0.6701 -1.1044 0.0776 0.0100, copy) search(norescale)
ml display
nlcom (delta: exp([LNdelta]_b[_cons])) (alpha: exp([LNalphaD]_b[_cons]))
nlcom (probEXP: 1/(1+exp([kappa]_b[_cons]))) (probHYP: exp([kappa]_b[_cons])/(1+exp([kappa]_b[_cons])))
nlcom (probSAME: 1/(1+exp([kappa]_b[_cons])) - exp([kappa]_b[_cons])/(1+exp([kappa]_b[_cons])))

* save data
save tmpMIXra, replace

* get RA exponential part of Figure 4
GetEx
twoway (rarea ExDisLO ExDisHI Horizon if Horizon>$HorizonVIEW & Horizon<=3, sort blcolor(gs14) bfcolor(gs14)) (connected ExDis Horizon if Horizon>$HorizonVIEW & Horizon<=3, msize(zero) sort lwidth(medthick)), title("A. Allowing Risk Aversion:" "Exponential Discounting (72%)", position(2) ring(0) justification(center) box) legend(off) ytitle("Discount Rate", size(medium)) xtitle("", size(vlarge)) xlabel(0(1)3, nogrid angle(horizontal)) ylabel(0(0.1)0.6 , nogrid angle(horizontal)) legend(off) saving(RAexp, replace)

* get RA hyperbolic part of Figure 4
use tmpMIXra, clear
GetHY
twoway (rarea HYDisLO HYDisHI Horizon if Horizon>$HorizonVIEW & Horizon<=3, sort blcolor(gs14) bfcolor(gs14)) (connected HYDis Horizon if Horizon>$HorizonVIEW & Horizon<=3, msize(zero) sort lwidth(medthick)), title("A. Allowing Risk Aversion:" "Hyperbolic Discounting (28%)", position(2) ring(0) justification(center) box) legend(off) ytitle("", size(medium) orientation(vertical)) xtitle("", size(vlarge)) ylabel(0(0.1)0.6, nogrid angle(horizontal))  legend(off) saving(RAhyp, replace)

* Figure 4 -- the RA bit
graph combine RAexp.gph RAhyp.gph, ycommon xcommon imargin(vsmall) col(2) b1title("Horizon in Years", position(6) ring(0)) saving(Figure4ra, replace)

* Figure 4 -- now put all the pieces together
graph combine Figure4ra.gph Figure4rn.gph, row(2) xcommon imargin(vsmall) title("Figure 4: Mixture Specifications of" "Exponential and Hyperbolic Discounting Models", size(vlarge)) subtitle(Estimated assuming linear utility or allowing concave utility) saving(Figure4, replace)

* return to data
use tmp, clear

* look at the distribution of choices in the last row of the MPL
tab choices if AR==50
    forvalues X=1(1)6 {
        tab choices if AR==50 & DRtask`X'==1
    }

log close mainlog
