/* AnalysisR.do implements the statistical estimation for the DK data with random coefficients */

* log just for random coefficients estimations
log using RiskAndTime_RandomCoefficients.log, replace text name(randylog)

* return to main data
use tmp, clear

* storage for predicted choices
capture generate Pchoices=.

* flag if we want the random effects to have zero mean (y) or not
global REzero_mean "y"

* flag if we do repeat estimation with more Halton draws
global DOrepeat "n"

* flag if we want to LOG the noise terms (y) or not in the RA estimation
global LOGnoise "n"

* do lognormal for RN case (y) or not
global DOlognormal "n"

* truncate normal to ensure non-negative delta
global DOtruncate "n"

* default algorithm
global alg "dfp"

* set characters
local Dalpha=char(225)
local Dbeta=char(226)
local Ddelta=char(228)
local Dlambda=char(235)
local Dgamma=char(227)
local Dmu=char(236)
local Dpi=char(240)

* specify default level of background consumption
global omega=118

* specify default lambda values for each task, where 0 refers to the early payment option (integer values only)
global lambda0=$DEFAULTlambda
global lambda1=$DEFAULTlambda
global lambda2=$DEFAULTlambda
global lambda3=$DEFAULTlambda
global lambda4=$DEFAULTlambda
global lambda5=$DEFAULTlambda
global lambda6=$DEFAULTlambda

* default to doing no truncation within ML (changed as needed by $DOtruncate) and not lognormal
global truncate "n"
global lognormal "n"

* identify a task variable so we only draw one random deviate per subject
bysort id ra: egen int task=seq()
tab task

* primes for Halton sequence
matrix z = (3 , 7)

* constrain random draws to be the same for the subject over all choices (y) or not
global random_same "n"

* Set smaller number of Halton draws
global SH   "10"
global SHra "10"

* Set larger number of Halton draws, if used
global LH   "15 20 25 30 35 40 45 50"
global LHra "15 20 25"

* program to do Halton draws
program define getHalton

* drop any Halton variables we might have in place already
capture {
    drop h1_* h2_*
}

* get the Halton sequence of uniform deviates
if "$random_same" == "y" {

    mdraws, draws($draw_raw) neq(2) prefix(h) replace burn(10) antithetics primes(z)
    summ h1_* h2_* if ra==1
    summ h1_* h2_* if ra==0

}

else {

    mdraws if task==1, draws($draw_raw) neq(2) prefix(h) replace burn(10) antithetics primes(z)

    * pick the first random uniform deviate for each subject and task type, and apply to all choices by that subject and task
    qui foreach T in 0 1 {
    qui forvalues v=1/2 {
    forvalues i=1/$draw {
        replace h`v'_`i'=. if task~=1 & ra==`T'  
        bysort id: egen rmax`T'=max(h`v'_`i') if ra==`T' 
        replace h`v'_`i'=rmax`T' if ra==`T'
        drop rmax`T'
    }
    }
    }
    summ h1_* h2_* if ra==1 & task==1
    summ h1_* h2_* if ra==0 & task==1

}
end

* save data prior to estimation stage
save tmp, replace

if "$DOrisk" == "rn" |  "$DOrisk" == "both" {

log using RiskAndTime_RandomCoefficients_RN.log, replace text name(randylogRN)

if "$DOlognormal" ~= "y" {

    global lognormal "n"

    *** ASSUME RISK NEUTRALITY
    * Table 3, without random effects
    ml model lf MLex_rn_lam (LNdelta: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (muIDR: ) if ra==0 , cluster(id) missing maximize technique(dfp) difficult init(-1.379 0.133, copy)
    ml display
    nlcom (delta: exp([LNdelta]_b[_cons]))

    local cons11 = [LNdelta]_cons
    local cons12 = [muIDR]_cons
    constraint define 11 [LNdelta]_cons =  `cons11'
    constraint define 12 [muIDR]_cons 	=  `cons12' 

    * Do constrained random effects with the previous solution
    global draw_raw=$SH
    global draw=$draw_raw*2
    qui getHalton

    display "Getting the initial starting values"
    ml model lf MLex_rn_lam_r (LNdelta: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (muIDR:  ) (LNdelta_sd:) if ra==0 , cluster(id) missing maximize technique($alg) difficult search(norescale) const(11 12) init(`cons11' `cons12' -3, copy) 
    ml display

   * Do the random effects model unconstrained on previous estimates
    display "Getting initial estimates with $draw Halton draws"
    ml model lf MLex_rn_lam_r (LNdelta: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (muIDR:  ) (LNdelta_sd:) if ra==0 , cluster(id) missing maximize technique($alg) difficult search(norescale) continue
    ml display 
    nlcom  (delta: exp([LNdelta]_b[_cons])) (delta_sd: exp([LNdelta_sd]_b[_cons])), iterate(10000)

    * now increment the number of Halton draws significantly and repeat the last estimation
    foreach X in $LH {

        global LH "`X'"
        global draw_raw=$LH
        global draw=$draw_raw*2
        qui getHalton

        display "Getting estimates with $draw Halton draws"
        ml model lf MLex_rn_lam_r (LNdelta: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (muIDR:  ) (LNdelta_sd:) if ra==0 , cluster(id) missing maximize technique($alg) difficult search(norescale) continue
        ml display 
        nlcom  (delta: exp([LNdelta]_b[_cons])) (delta_sd: exp([LNdelta_sd]_b[_cons])), iterate(10000)
    
        * Save values for figure 2B
        scalar RNmean = exp([LNdelta]_cons)
        scalar RNsd   = exp([LNdelta_sd]_cons)

    }

    * add this code if we want to truncate the normal, assuming same Halton draws as last pass above
    if "$DOtruncate" == "y" {

        * now let the ML code truncate
        global truncate "y"

        * initialize
        local c1 = -1.33934
        local c2 = .0564561
        local c3 = -1.761763

        * solve
        display "Getting estimates with $draw Halton draws with truncation"
        ml model lf MLex_rn_lam_r (LNdelta: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (muIDR:  ) (LNdelta_sd:) if ra==0 , cluster(id) missing maximize technique($alg) difficult search(norescale) init(`c1' `c2' `c3', copy)
        ml display 
        nlcom  (delta: exp([LNdelta]_b[_cons])) (delta_sd: exp([LNdelta_sd]_b[_cons])), iterate(10000)

        * Save values for figure 2B
        scalar RNmean = exp([LNdelta]_cons)
        scalar RNsd   = exp([LNdelta_sd]_cons)

        * re-set this to not being operative
        global truncate "n"

    }

}

    if "$DOlognormal" == "y" {

        global lognormal "y"

        ** Now do lognormal distribution
        ml model lf MLex_rn_lam (LNdelta: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (muIDR: ) if ra==0 , cluster(id) missing maximize technique(dfp) difficult init(-1.379 0.133, copy)
        ml display
        nlcom (delta: exp([LNdelta]_b[_cons]))

        local cons11 = [LNdelta]_cons
        local cons12 = [muIDR]_cons
        constraint define 11 [LNdelta]_cons =  `cons11'
        constraint define 12 [muIDR]_cons 	=  `cons12' 

        global draw_raw=$SH
        global draw=$draw_raw*2
        qui getHalton

        * Do constrained random effects with the previous solution
        global draw_raw=$SH
        global draw=$draw_raw*2
        qui getHalton

        display "Getting the initial starting values"
        ml model lf MLex_rn_lam_r (LNdelta: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (muIDR:  ) (LNdelta_sd:) if ra==0 , cluster(id) missing maximize technique($alg) difficult search(norescale) const(11 12) init(`cons11' `cons12' -3, copy) 
        ml display

        * Do the random effects model unconstrained on previous estimates
        display "Getting initial estimates with $draw Halton draws"
        ml model lf MLex_rn_lam_r (LNdelta: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (muIDR:  ) (LNdelta_sd:) if ra==0 , cluster(id) missing maximize technique($alg) difficult search(norescale) continue
        ml display 
        nlcom  (delta: exp([LNdelta]_b[_cons])) (delta_sd: exp([LNdelta_sd]_b[_cons])), iterate(10000)

        * now increment the number of Halton draws significantly and repeat the last estimation
        foreach X in $LH {
            global LH "`X'"
            global draw_raw=$LH
            global draw=$draw_raw*2
            qui getHalton

            display "Getting estimates with $draw Halton draws"
            ml model lf MLex_rn_lam_r (LNdelta: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (muIDR:  ) (LNdelta_sd:) if ra==0 , cluster(id) missing maximize technique($alg) difficult search(norescale) continue
            ml display 
            nlcom  (delta: exp([LNdelta]_b[_cons])) (delta_sd: exp([LNdelta_sd]_b[_cons])), iterate(10000)
    
            scalar RNmean = exp([LNdelta]_cons)
            scalar RNsd   = exp([LNsigma_d]_cons)
        }

    }

    log close randylogRN
    
}


if "$DOrisk" == "ra" |  "$DOrisk" == "both" {

log using RiskAndTime_RandomCoefficients_RA.log, replace text name(randylogRA)

** ASSUME RISK AVERSION

    * return to default to not using lognormal
    global lognormal "n"

    use tmp, clear

    * re-estimate the basic model
    ml model lf MLex_ra_crra_lam (r: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (LNdelta: ) (muRA: ) (muIDR: ), cluster(id) missing maximize technique(dfp) difficult init(0.746 -2.002 0.084 0.017, copy)
    ml display

    * save values, converting noise parameters to logs
    local c1 = [r]_cons
    local c2 = [LNdelta]_cons
    if "$LOGnoise" == "y" {
        local c3 = ln([muRA]_cons)
        local c4 = ln([muIDR]_cons)
    }
    else {
        local c3 = [muRA]_cons
        local c4 = [muIDR]_cons
    }

    * constrain the terms other than the random effects
    constraint define 1 [r]_cons = `c1'
    constraint define 2 [LNdelta]_cons = `c2'
    constraint define 3 [LNmuRA]_cons = `c3'
    constraint define 4 [LNmuIDR]_cons = `c4'
    constraint define 5 [muRA]_cons = `c3'
    constraint define 6 [muIDR]_cons = `c4'
    constraint list

    * get the Halton draws set up
    global draw_raw=$SHra
    global draw=$draw_raw*2
    qui getHalton

    * start from same starting values as non-random model
    display "Getting zero-mean RE estimates with $draw Halton draws"
    if "$LOGnoise" == "y" {
        ml model lf MLex_ra_crra_lam_r_ (r: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (LNdelta: ) (LNmuRA: ) (LNmuIDR: ) (LNr_sd: ) (LNdelta_sd:) (atrho12: ), cluster(id) missing maximize technique($alg) difficult init(`c1' `c2' `c3' `c4' -3 -3 0, copy) constraints(1 2 3 4) search(norescale)
    }
    else {
        ml model lf MLex_ra_crra_lam_r (r: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (LNdelta: ) (muRA: ) (muIDR: ) (LNr_sd: ) (LNdelta_sd:) (atrho12: ), cluster(id) missing maximize technique($alg) difficult init(`c1' `c2' `c3' `c4' -3 -3 0, copy) constraints(1 2 5 6) search(norescale)
    }
    ml display
    nlcom  (r_sd: exp([LNr_sd]_b[_cons])) (delta: exp([LNdelta]_b[_cons])) (delta_sd: exp([LNdelta_sd]_b[_cons])) (corr_rd: tanh([atrho12]_b[_cons]) ),  iterate(10000)

    
    * now increment the number of Halton draws and repeat the last estimation
    if "$DOrepeat" == "y" {

    foreach X in $LHra {

        global LH "`X'"
        global draw_raw=$LH
        global draw=$draw_raw*2
        qui getHalton

        display "Getting estimates with $draw Halton draws"
        if "$LOGnoise" == "y" {
            ml model lf MLex_ra_crra_lam_r_ (r: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (LNdelta: ) (LNmuRA: ) (LNmuIDR: ) (LNr_sd: ) (LNdelta_sd:) (atrho12: ), cluster(id) missing maximize technique(bfgs) difficult continue constraints(1 2 3 4) search(norescale)
         }
         else {
            ml model lf MLex_ra_crra_lam_r (r: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (LNdelta: ) (muRA: ) (muIDR: ) (LNr_sd: ) (LNdelta_sd:) (atrho12: ), cluster(id) missing maximize technique(bfgs) difficult continue constraints(1 2 5 6) search(norescale)
         }
         ml display 
         nlcom  (delta: exp([LNdelta]_b[_cons])) (delta_sd: exp([LNdelta_sd]_b[_cons])), iterate(10000)
    
    }

    }

    * now unconstrained if requested
    if "$REzero_mean" ~= "y" {

        if "$LOGnoise" == "y" {
            ml model lf MLex_ra_crra_lam_r_ (r: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (LNdelta: ) (LNmuRA: ) (LNmuIDR: ) (LNr_sd: ) (LNdelta_sd:) (atrho12: ), cluster(id) missing maximize technique($alg) difficult continue search(norescale)
        }
        else {
            ml model lf MLex_ra_crra_lam_r (r: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (LNdelta: ) (muRA: ) (muIDR: ) (LNr_sd: ) (LNdelta_sd:) (atrho12: ), cluster(id) missing maximize technique($alg) difficult continue search(norescale)
        }
        ml display
        nlcom  (r_sd: exp([LNr_sd]_b[_cons])) (delta: exp([LNdelta]_b[_cons])) (delta_sd: exp([LNdelta_sd]_b[_cons])) (corr_rd: tanh([atrho12]_b[_cons]) ),  iterate(10000)

        scalar RAmean = exp([LNdelta]_cons)
        scalar RAsd   = exp([LNdelta_sd]_cons)

        local RAmean = round(exp([LNdelta]_cons), 0.01)
        local RAsd   = round(exp([LNdelta_sd]_cons), 0.01)
        save tmp, replace

    }

    * add this code if we want to truncate the normal
    if "$DOtruncate" == "y" {

        * now let the ML code truncate
        global truncate "y"

        * solve
        display "Getting estimates on high numbers of Halton draws with truncated distribution on delta"
        if "$LOGnoise" == "y" {
            ml model lf MLex_ra_crra_lam_r_ (r: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (LNdelta: ) (LNmuRA: ) (LNmuIDR: ) (LNr_sd: ) (LNdelta_sd:) (atrho12: ), cluster(id) missing maximize technique($alg) difficult search(norescale) continue trace
        }
        else {
            ml model lf MLex_ra_crra_lam_r (r: choices horizon principal DelayedP RowProbA ra Vprizea1 Vprizea2 Vprizeb1 Vprizeb2 DRtask1 DRtask2 DRtask3 DRtask4 DRtask5 DRtask6 = ) (LNdelta: ) (muRA: ) (muIDR: ) (LNr_sd: ) (LNdelta_sd:) (atrho12: ), cluster(id) missing maximize technique($alg) difficult search(norescale) continue trace
        }
        ml display 
        nlcom  (r_sd: exp([LNr_sd]_b[_cons])) (delta: exp([LNdelta]_b[_cons])) (delta_sd: exp([LNdelta_sd]_b[_cons])) (corr_rd: tanh([atrho12]_b[_cons]) ),  iterate(10000)
    
        scalar RAmean = exp([LNdelta]_cons)
        scalar RAsd   = exp([LNdelta_sd]_cons)

        * re-set this to not being operative
        global truncate "n"

    }

log close randylogRA

}


log close randylog
