/* Data.do reads in all the data */

use RiskAndTime.dta, clear

*	Variable definitions

* id		subject id
* age		age in years
* male		gender (male=1, female=0)
* live		where does the subject live (copenhagen, town, village...)
* county	county code
* residence	type of residence the subject lives in (owner/rented, house/apartment..)
* occupation	primary occupation during the last 12 months
* education	highest education level
* education1	vocational education and training
* education2	short higher education
* education3	medium higher education
* education4	long higher education
* hhd		household characteristics (singles/households,children...)
* nhhd		number of people in household
* income	total income of the household earned in 2002 divided into intervals
* cigs		does the subject smoke (no=0, yes=1)
* ncigs		the number of cigarettes the subject smokes in one day
* Task1		first risk aversion task
* Task2		second risk aversion task
* Task3		third risk aversion task
* Task4		fourth risk aversion task
* Rtask		risk aversion task (first task=4, second task=5, third task=6, fourth task=7)
* choiceLO	lower bound on probability interval
* choiceHI	higher bound on probability interval
* choice	midpoint of probability interval
* crraLO	lower bound on CRRA interval
* crraHI	higher bound on CRRA interval
* crra		midpoint of CRRA interval
* crraDIFF	CRRA interval size
* n_a		number of safe choices
* n_i		number of indifferent choices
* n_b		number of risky choices
* n_a_lo	aggregate number of safe choices in symmetric treatment adjusted to skewLO intervals
* n_a_hi	aggregate number of safe choices in symmetric treatment adjusted to skewHI intervals
* RAtreat	framing effect (skewLO=-1, symmetric=0, skewHI=1)
* RApay		RA payments
* Final		final response
* Initial	initial response
* experimenter	experimenter effect (Lau=0 and Andersen=1)
* income	total income of the household earned in 2002 divided into intervals
* cigs		does the subject smoke (no=0, yes=1)
* ncigs		the number of cigarettes the subject smokes in one day
* check		does the subject have a checking account (no=0, yes=1)
* c_rate	annual interest rate on the checking account (don't know=88, else=x%)
* c_bal		balance on the cheking account divided into intervals (don't know=8)
* loc		does the subject have a line of credit (no=0, yes=1)
* loc_rate	annual interest rate on the line of credit (don't know=88, else=x%)
* loc_carry	subject ordinarily carries a balance from month to month (no=0, yes=1)
* loc_bal	balance owed on the line of credit - divided into intervals (don't know=8)
* cc		does the subject have a credit card (no=0, yes=1)
* cc_rate	annual interest rate on the credit card (don't know=88, else=x%)
* cc_bal	balance owed on the credit card, divided into intervals (don't know=8)
* cc2		does the subject have a second credit card (no=0, yes=1)
* cc2_rate	annual interest rate on the second credit card (don't know=88, else=x%)
* cc2_bal	balance owed on the second credit card, divided into intervals (don't know=8)
* loan		does the subject have a student loan (no=0, yes=1)
* loan_rate	annual interest rate on the student loan (don't know=88, else=x%)
* loan_bal	balance owed on the student loan, divided into intervals (don't know=8)
* sav		does the subject have a savings account (no=0, yes=1)
* sav_rate	annual interest rate on the savings account (don't know=88, else=x%)
* sav_bal	balance on the savings account, divided into intervals (don't know=8)
* inv		does the subject have other investment accounts (no=0, yes=1)
* inv_rate	(highest) interest rate on the investment accounts (don't know=88, else=x%)
* inv_bal	balance on that investment account, divided into intervals (don't know=8)
* chances	chances to obtain a loan... (at least 90% likely=1, 75%=2, 50%=3, less=4)
* short		short of cash between paychecks (every time=1, 3/4=2,.., almost never=5)
* horizon1	1 month horizon
* horizon4	4 months horizon
* horizon6	6 months horizon
* horizon12	12 months horizon
* horizon18	18 months horizon
* horizon24	24 months horizon
* idrLO		lower bound on IDR interval
* idrHI		higher bound on IDR interval
* idr		midpoint of IDR interval
* idrDIFF	IDR interval size

** ==============================================================================
* Generalize treatments

* horizon dummies

foreach X in 1 4 6 12 18 24 {
	replace horizon`X'=0 if horizon`X'==.
}

#delimit ;

keep 
/* basic information */

id ra choice	

/* IDR information */

horizon* DRtask* AR* AER* principal DelayedP	

/* RA information */

Vprize* RowP* Rtask*

/* treatments */

endowment RApay IDRpay experimenter  skewLO skewHI 

/* Demographics general*/

female young middle old student copen city owner retired skilled longedu kids single county weight IncHigh IncLow;

# delimit cr

* save data
save Data, replace

exit