/* MLfunctions.do reads in all the handwritten ml functions */

* Overview of models *

*	ML_crra			Risk attitudes only with CRRA
*	MLex_rn_lam		Exponential model model with risk neutrality
*	MLh_rn_lam		Mazur-hyperbolic model with risk neutrality
*	MLp_rn_lam		Prelec-hyperbolic model with risk neutrality
*	MLex_ra_crra_lam	Exponential model with CRRA
*	MLh_ra_crra_lam		Mazur-hyperbolic model with CRRA
*	MLp_ra_crra_lam		Prelec-hyperbolic model with CRRA
*	MLmix_ex_h_rn_lam	Mixture model of exponential and Mazur-hyperbolic discounting with risk neutrality
*	MLmix_ex_h_lam		Mixture model of exponential and Mazur-hyperbolic discounting with CRRA


** RISK ATTITUDES ONLY **

* CRRA specification
program ML_crra

    args lnf r LNw muRA

    tempvar ra t mA mB prob1 prob2 mA1 mA2 mB1 mB2 yA1 yA2 yB1 yB2 wp1 wp2 euA euB euRatio y0 y1 
    tempvar t mA mB uA uB uRatio baseSR baseMR rMR scale delta w

	syntax varlist [if] [in] [fweight pweight] [, noLOg	Robust CLuster(varname)	svy	EForm *]
	marksample touse
	if "`svy'" != "" {
		svymarkout `touse'
	}
 
    quietly {

        * initialize horizon and payoffs
        generate double `ra' 	= $ML_y6
        generate double `t' 	= $ML_y2/360
        generate double `mA'	= $ML_y3
        generate double `mB'	= $ML_y4	
        generate double `prob1' = $ML_y5      
        generate double `prob2' = 1 - `prob1'
        generate double `mA1' 	= $ML_y7      
        generate double `mA2' 	= $ML_y8
        generate double `mB1' 	= $ML_y9
        generate double `mB2' 	= $ML_y10

        * transform parameters
        generate double `w'     = exp(`LNw')

        * estimating over risk responses
        generate double `yA1' = ((`w'+`mA1')^(1-`r'))/(1-`r')	if `ra'==1
        generate double `yA2' = ((`w'+`mA2')^(1-`r'))/(1-`r')	if `ra'==1
        generate double `yB1' = ((`w'+`mB1')^(1-`r'))/(1-`r')	if `ra'==1
        generate double `yB2' = ((`w'+`mB2')^(1-`r'))/(1-`r')	if `ra'==1

        replace `yA1' = ln(`w'+`mA1')				if `ra'==1 & `r'==1
        replace `yA2' = ln(`w'+`mA2')				if `ra'==1 & `r'==1
        replace `yB1' = ln(`w'+`mB1')				if `ra'==1 & `r'==1
        replace `yB2' = ln(`w'+`mB2')				if `ra'==1 & `r'==1

        generate double `wp1' = `prob1'								if `ra'==1	
        generate double `wp2' = `prob2'								if `ra'==1
        generate double `euA' = (`wp1'*`yA1')+(`wp2'*`yA2')					if `ra'==1      
        generate double `euB' = (`wp1'*`yB1')+(`wp2'*`yB2')					if `ra'==1

	* generate index
        generate double `euRatio' = (`euA'^(1/`muRA'))/((`euA'^(1/`muRA'))+(`euB'^(1/`muRA')))	if `ra'==1

        replace `lnf' = ln(`euRatio')						if $ML_y1==0	&  `ra'==1
        replace `lnf' = ln(1-`euRatio')						if $ML_y1==1	&  `ra'==1
        replace `lnf' = (ln(1-`euRatio')/2)+(ln(`euRatio')/2)			if $ML_y1==-1	&  `ra'==1

    }

end


** RISK NEUTRALITY **
* Exponential model with risk neutrality
program MLex_rn_lam

    args lnf LNdelta muIDR

    tempvar t mA mB y0 y1 tau uRatio delta task1 task2 task3 task4 task5 task6

    * Define temporary variables
	forvalues X = 1(1)$lambda0 {
	    tempvar uA_S_`X'
	}
	forvalues X = 1(1)$lambda1 {
	    tempvar uB_L1_`X'
	}
	forvalues X = 1(1)$lambda2 {
	    tempvar uB_L2_`X'
	}
	forvalues X = 1(1)$lambda3 {
	    tempvar uB_L3_`X'
	}
	forvalues X = 1(1)$lambda4 {
	    tempvar uB_L4_`X'
	}
	forvalues X = 1(1)$lambda5 {
	    tempvar uB_L5_`X'
	}
	forvalues X = 1(1)$lambda6 {
	    tempvar uB_L6_`X'
	}

	syntax varlist [if] [in] [fweight pweight] [, noLOg	Robust CLuster(varname)	svy	EForm *]
	marksample touse
	if "`svy'" != "" {
		svymarkout `touse'
	}
 
    quietly {

        * initialize horizon and payoffs
        generate double `t' 	= $ML_y2/360
        generate double `tau' 	= (30+$ML_y2)/360
        generate double `mA'	= $ML_y3
        generate double `mB'	= $ML_y4	
	generate double `task1' = $ML_y11
	generate double `task2' = $ML_y12
	generate double `task3' = $ML_y13
	generate double `task4' = $ML_y14
	generate double `task5' = $ML_y15
	generate double `task6' = $ML_y16

        * transform parameters
        generate double `delta' = exp(`LNdelta')

        * estimating over time responses
        generate double `y0' = 0
        generate double `y1' = 0

	forvalues X = 1(1)$lambda0 {
	    generate double `uA_S_`X'' = (1 / ((1+`delta')^(30/360 + (`X'-1)/360 )) ) * (`mA'/$lambda0)

	    replace `y0' = `y0' + `uA_S_`X''
	}	

	forvalues X = 1(1)$lambda1 {
	    generate double `uB_L1_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (`mB'/$lambda1)		if `task1'==1

	    replace `y1' = `y1'  +  `uB_L1_`X''		if `task1'==1
	}	

	forvalues X = 1(1)$lambda2 {
	    generate double `uB_L2_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (`mB'/$lambda2)		if `task2'==1

	    replace `y1' = `y1'  +  `uB_L2_`X''		if `task2'==1
	}	

	forvalues X = 1(1)$lambda3 {
	    generate double `uB_L3_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (`mB'/$lambda3)		if `task3'==1

	    replace `y1' = `y1'  +  `uB_L3_`X''		if `task3'==1
	}	

	forvalues X = 1(1)$lambda4 {
	    generate double `uB_L4_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (`mB'/$lambda4)		if `task4'==1

	    replace `y1' = `y1'  +  `uB_L4_`X''		if `task4'==1
	}	

	forvalues X = 1(1)$lambda5 {
	    generate double `uB_L5_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (`mB'/$lambda5)		if `task5'==1

	    replace `y1' = `y1'  +  `uB_L5_`X''		if `task5'==1
	}	

	forvalues X = 1(1)$lambda6 {
	    generate double `uB_L6_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (`mB'/$lambda6)		if `task6'==1

	    replace `y1' = `y1'  +  `uB_L6_`X''		if `task6'==1
	}	

        generate double `uRatio' = (`y0'^(1/`muIDR'))/((`y0'^(1/`muIDR'))+(`y1'^(1/`muIDR')))

        replace `lnf' = ln(`uRatio')					if 	$ML_y1==0
        replace `lnf' = ln(1-`uRatio')					if 	$ML_y1==1
        replace `lnf' = (ln(1-`uRatio')/2)+(ln(`uRatio')/2)		if	$ML_y1==-1

    }

end


* Mazur-hyperbolic model with risk neutrality
program MLh_rn_lam

    args lnf LNalphaD muIDR

    tempvar t mA mB y0 y1 tau uRatio alphaD task1 task2 task3 task4 task5 task6

    * Define temporary variables
	forvalues X = 1(1)$lambda0 {
	    tempvar uA_S_`X'
	}
	forvalues X = 1(1)$lambda1 {
	    tempvar uB_L1_`X'
	}
	forvalues X = 1(1)$lambda2 {
	    tempvar uB_L2_`X'
	}
	forvalues X = 1(1)$lambda3 {
	    tempvar uB_L3_`X'
	}
	forvalues X = 1(1)$lambda4 {
	    tempvar uB_L4_`X'
	}
	forvalues X = 1(1)$lambda5 {
	    tempvar uB_L5_`X'
	}
	forvalues X = 1(1)$lambda6 {
	    tempvar uB_L6_`X'
	}

	syntax varlist [if] [in] [fweight pweight] [, noLOg	Robust CLuster(varname)	svy	EForm *]
	marksample touse
	if "`svy'" != "" {
		svymarkout `touse'
	}
 
    quietly {

        * initialize horizon and payoffs
        generate double `t' 	= $ML_y2/360
        generate double `tau' 	= (30+$ML_y2)/360
        generate double `mA'	= $ML_y3
        generate double `mB'	= $ML_y4	
	generate double `task1' = $ML_y11
	generate double `task2' = $ML_y12
	generate double `task3' = $ML_y13
	generate double `task4' = $ML_y14
	generate double `task5' = $ML_y15
	generate double `task6' = $ML_y16

        * transform parameters
        generate double `alphaD' = exp(`LNalphaD')

        * estimating over time responses
        generate double `y0' = 0
        generate double `y1' = 0

	forvalues X = 1(1)$lambda0 {
	    generate double `uA_S_`X''  = (1 / (1+`alphaD'*(30/360 + (`X'-1)/360)) ) * (`mA'/$lambda0)

	    replace `y0' = `y0' + `uA_S_`X''
	}	

	forvalues X = 1(1)$lambda1 {
	    generate double `uB_L1_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360)) ) * (`mB'/$lambda1)		if `task1'==1

	    replace `y1' = `y1'  +  `uB_L1_`X''		if `task1'==1
	}	

	forvalues X = 1(1)$lambda2 {
	    generate double `uB_L2_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360)) ) * (`mB'/$lambda2)		if `task2'==1

	    replace `y1' = `y1'  +  `uB_L2_`X''		if `task2'==1
	}	

	forvalues X = 1(1)$lambda3 {
	    generate double `uB_L3_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360)) ) * (`mB'/$lambda3)		if `task3'==1

	    replace `y1' = `y1'  +  `uB_L3_`X''		if `task3'==1
	}	

	forvalues X = 1(1)$lambda4 {
	    generate double `uB_L4_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360)) ) * (`mB'/$lambda4)		if `task4'==1

	    replace `y1' = `y1'  +  `uB_L4_`X''		if `task4'==1
	}	

	forvalues X = 1(1)$lambda5 {
	    generate double `uB_L5_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360)) ) * (`mB'/$lambda5)		if `task5'==1

	    replace `y1' = `y1'  +  `uB_L5_`X''		if `task5'==1
	}	

	forvalues X = 1(1)$lambda6 {
	    generate double `uB_L6_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360)) ) * (`mB'/$lambda6)		if `task6'==1

	    replace `y1' = `y1'  +  `uB_L6_`X''		if `task6'==1
	}	

        generate double `uRatio' = (`y0'^(1/`muIDR'))/((`y0'^(1/`muIDR'))+(`y1'^(1/`muIDR')))

        replace `lnf' = ln(`uRatio')					if 	$ML_y1==0
        replace `lnf' = ln(1-`uRatio')					if 	$ML_y1==1
        replace `lnf' = (ln(1-`uRatio')/2)+(ln(`uRatio')/2)		if	$ML_y1==-1

    }

end


* Prelec-hyperbolic model with risk neutrality
program MLp_rn_lam

    args lnf betaD alphaD_ muIDR

    tempvar t mA mB y0 y1 tau uRatio task1 task2 task3 task4 task5 task6

    * Define temporary variables
	forvalues X = 1(1)$lambda0 {
	    tempvar uA_S_`X'
	}
	forvalues X = 1(1)$lambda1 {
	    tempvar uB_L1_`X'
	}
	forvalues X = 1(1)$lambda2 {
	    tempvar uB_L2_`X'
	}
	forvalues X = 1(1)$lambda3 {
	    tempvar uB_L3_`X'
	}
	forvalues X = 1(1)$lambda4 {
	    tempvar uB_L4_`X'
	}
	forvalues X = 1(1)$lambda5 {
	    tempvar uB_L5_`X'
	}
	forvalues X = 1(1)$lambda6 {
	    tempvar uB_L6_`X'
	}

	syntax varlist [if] [in] [fweight pweight] [, noLOg	Robust CLuster(varname)	svy	EForm *]
	marksample touse
	if "`svy'" != "" {
		svymarkout `touse'
	}
 
    quietly {

        * initialize horizon and payoffs
        generate double `t' 	= $ML_y2/360
        generate double `tau' 	= (30+$ML_y2)/360
        generate double `mA'	= $ML_y3
        generate double `mB'	= $ML_y4	
	generate double `task1' = $ML_y11
	generate double `task2' = $ML_y12
	generate double `task3' = $ML_y13
	generate double `task4' = $ML_y14
	generate double `task5' = $ML_y15
	generate double `task6' = $ML_y16

        * estimating over time responses
        generate double `y0' = 0
        generate double `y1' = 0

	forvalues X = 1(1)$lambda0 {
	    generate double `uA_S_`X''  = exp(-`betaD'*(30/360 + (`X'-1)/360)^(`alphaD_')) * (`mA'/$lambda0)

	    replace `y0' = `y0' + `uA_S_`X''
	}	

	forvalues X = 1(1)$lambda1 {
	    generate double `uB_L1_`X'' = exp(-`betaD'*(`tau' + (`X'-1)/360)^(`alphaD_')) * (`mB'/$lambda1)	if `task1'==1

	    replace `y1' = `y1'  +  `uB_L1_`X''		if `task1'==1
	}	

	forvalues X = 1(1)$lambda2 {
	    generate double `uB_L2_`X'' = exp(-`betaD'*(`tau' + (`X'-1)/360)^(`alphaD_')) * (`mB'/$lambda2)	if `task2'==1

	    replace `y1' = `y1'  +  `uB_L2_`X''		if `task2'==1
	}	

	forvalues X = 1(1)$lambda3 {
	    generate double `uB_L3_`X'' = exp(-`betaD'*(`tau' + (`X'-1)/360)^(`alphaD_')) * (`mB'/$lambda3)	if `task3'==1

	    replace `y1' = `y1'  +  `uB_L3_`X''		if `task3'==1
	}	

	forvalues X = 1(1)$lambda4 {
	    generate double `uB_L4_`X'' = exp(-`betaD'*(`tau' + (`X'-1)/360)^(`alphaD_')) * (`mB'/$lambda4)	if `task4'==1

	    replace `y1' = `y1'  +  `uB_L4_`X''		if `task4'==1
	}	

	forvalues X = 1(1)$lambda5 {
	    generate double `uB_L5_`X'' = exp(-`betaD'*(`tau' + (`X'-1)/360)^(`alphaD_')) * (`mB'/$lambda5)	if `task5'==1

	    replace `y1' = `y1'  +  `uB_L5_`X''		if `task5'==1
	}	

	forvalues X = 1(1)$lambda6 {
	    generate double `uB_L6_`X'' = exp(-`betaD'*(`tau' + (`X'-1)/360)^(`alphaD_')) * (`mB'/$lambda6)	if `task6'==1

	    replace `y1' = `y1'  +  `uB_L6_`X''		if `task6'==1
	}	

        generate double `uRatio' = (`y0'^(1/`muIDR'))/((`y0'^(1/`muIDR'))+(`y1'^(1/`muIDR')))

        replace `lnf' = ln(`uRatio')					if 	$ML_y1==0
        replace `lnf' = ln(1-`uRatio')					if 	$ML_y1==1
        replace `lnf' = (ln(1-`uRatio')/2)+(ln(`uRatio')/2)		if	$ML_y1==-1

    }

end



** JOINT RISK AND TIME ESTIMATION **

* Exponential model with CRRA
program MLex_ra_crra_lam

    args lnf r LNdelta muRA muIDR

    tempvar ra t mA mB prob1 prob2 mA1 mA2 mB1 mB2 yA1 yA2 yB1 yB2 wp1 wp2 euA euB euRatio y0 y1 
    tempvar tau mA mB uA uB uRatio baseSR baseMR rMR scale delta lam0 lam1 lambda1 lambda2
    tempvar task1 task2 task3 task4 task5 task6

    * Define temporary variables
	forvalues X = 1(1)$lambda0 {
	    tempvar uA_S_`X' uB_S_`X'
	}
	forvalues X = 1(1)$lambda1 {
	    tempvar uA_L1_`X' uB_L1_`X'
	}
	forvalues X = 1(1)$lambda2 {
	    tempvar uA_L2_`X' uB_L2_`X'
	}
	forvalues X = 1(1)$lambda3 {
	    tempvar uA_L3_`X' uB_L3_`X'
	}
	forvalues X = 1(1)$lambda4 {
	    tempvar uA_L4_`X' uB_L4_`X'
	}
	forvalues X = 1(1)$lambda5 {
	    tempvar uA_L5_`X' uB_L5_`X'
	}
	forvalues X = 1(1)$lambda6 {
	    tempvar uA_L6_`X' uB_L6_`X'
	}

	syntax varlist [if] [in] [fweight pweight] [, noLOg	Robust CLuster(varname)	svy	EForm *]
	marksample touse
	if "`svy'" != "" {
		svymarkout `touse'
	}
 
    quietly {

        * initialize horizon and payoffs
        generate double `ra' 	= $ML_y6
        generate double `t' 	= $ML_y2/360
        generate double `tau' 	= (30+$ML_y2)/360
        generate double `mA'	= $ML_y3
        generate double `mB'	= $ML_y4	
        generate double `prob1' = $ML_y5      
        generate double `prob2' = 1 - `prob1'
        generate double `mA1' 	= $ML_y7      
        generate double `mA2' 	= $ML_y8
        generate double `mB1' 	= $ML_y9
        generate double `mB2' 	= $ML_y10
	generate double `task1' = $ML_y11
	generate double `task2' = $ML_y12
	generate double `task3' = $ML_y13
	generate double `task4' = $ML_y14
	generate double `task5' = $ML_y15
	generate double `task6' = $ML_y16

        * transform parameters
        generate double `delta' = exp(`LNdelta')

        * estimating over risk responses
        generate double `yA1' = (($omega+`mA1')^(1-`r'))/(1-`r')	if `ra'==1
        generate double `yA2' = (($omega+`mA2')^(1-`r'))/(1-`r')	if `ra'==1
        generate double `yB1' = (($omega+`mB1')^(1-`r'))/(1-`r')	if `ra'==1
        generate double `yB2' = (($omega+`mB2')^(1-`r'))/(1-`r')	if `ra'==1

        generate double `wp1' = `prob1'					if `ra'==1	
        generate double `wp2' = `prob2'					if `ra'==1
        generate double `euA' = (`wp1'*`yA1')+(`wp2'*`yA2')		if `ra'==1      
        generate double `euB' = (`wp1'*`yB1')+(`wp2'*`yB2')		if `ra'==1

        generate double `euRatio' = (`euA'^(1/`muRA'))/((`euA'^(1/`muRA'))+(`euB'^(1/`muRA')))	if `ra'==1

        * save predicted RA results
        replace Pchoices=0 if `euA'>`euB' & `ra'==1
        replace Pchoices=1 if `euA'<`euB' & `ra'==1

        replace `lnf' = ln(`euRatio')					if $ML_y1==0	& `ra'==1
        replace `lnf' = ln(1-`euRatio')					if $ML_y1==1	& `ra'==1
        replace `lnf' = (ln(1-`euRatio')/2)+(ln(`euRatio')/2)		if $ML_y1==-1	& `ra'==1

        * estimating over time responses
        generate double `y0' = 0
        generate double `y1' = 0

	* Exponential discounting
	forvalues X = 1(1)$lambda0 {
	    generate double `uA_S_`X'' = (1 / ((1+`delta')^(30/360 + (`X'-1)/360 )) ) * (($omega + `mA'/$lambda0)^(1-`r'))/(1-`r')	if `ra'==0
	    generate double `uB_S_`X'' = (1 / ((1+`delta')^(30/360 + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0

	    replace `y0' = `y0' +  `uA_S_`X''		if `ra'==0
	    replace `y1' = `y1' +  `uB_S_`X''		if `ra'==0
	}	

	forvalues X = 1(1)$lambda1 {
	    generate double `uA_L1_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task1'==1
	    generate double `uB_L1_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda1)^(1-`r'))/(1-`r')	if `ra'==0 & `task1'==1

	    replace `y0' = `y0'  +  `uA_L1_`X''		if `ra'==0 & `task1'==1
	    replace `y1' = `y1'  +  `uB_L1_`X''		if `ra'==0 & `task1'==1
	}	

	forvalues X = 1(1)$lambda2 {
	    generate double `uA_L2_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task2'==1
	    generate double `uB_L2_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda2)^(1-`r'))/(1-`r')	if `ra'==0 & `task2'==1

	    replace `y0' = `y0'  +  `uA_L2_`X''		if `ra'==0 & `task2'==1
	    replace `y1' = `y1'  +  `uB_L2_`X''		if `ra'==0 & `task2'==1
	}	

	forvalues X = 1(1)$lambda3 {
	    generate double `uA_L3_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task3'==1
	    generate double `uB_L3_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda3)^(1-`r'))/(1-`r')	if `ra'==0 & `task3'==1

	    replace `y0' = `y0'  +  `uA_L3_`X''		if `ra'==0 & `task3'==1
	    replace `y1' = `y1'  +  `uB_L3_`X''		if `ra'==0 & `task3'==1
	}	

	forvalues X = 1(1)$lambda4 {
	    generate double `uA_L4_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task4'==1
	    generate double `uB_L4_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda4)^(1-`r'))/(1-`r')	if `ra'==0 & `task4'==1

	    replace `y0' = `y0'  +  `uA_L4_`X''		if `ra'==0 & `task4'==1
	    replace `y1' = `y1'  +  `uB_L4_`X''		if `ra'==0 & `task4'==1
	}	

	forvalues X = 1(1)$lambda5 {
	    generate double `uA_L5_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task5'==1
	    generate double `uB_L5_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda5)^(1-`r'))/(1-`r')	if `ra'==0 & `task5'==1

	    replace `y0' = `y0'  +  `uA_L5_`X''		if `ra'==0 & `task5'==1
	    replace `y1' = `y1'  +  `uB_L5_`X''		if `ra'==0 & `task5'==1
	}	

	forvalues X = 1(1)$lambda6 {
	    generate double `uA_L6_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task6'==1
	    generate double `uB_L6_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda6)^(1-`r'))/(1-`r')	if `ra'==0 & `task6'==1

	    replace `y0' = `y0'  +  `uA_L6_`X''		if `ra'==0 & `task6'==1
	    replace `y1' = `y1'  +  `uB_L6_`X''		if `ra'==0 & `task6'==1
	}	

        * save DR results
        replace Pchoices=0 if `y0'>`y1' & `ra'==0
        replace Pchoices=1 if `y0'<`y1' & `ra'==0

        generate double `uRatio' = (`y0'^(1/`muIDR'))/((`y0'^(1/`muIDR'))+(`y1'^(1/`muIDR'))) 	if `ra'==0

        replace `lnf' = ln(`uRatio')					if 	$ML_y1==0	& `ra'==0
        replace `lnf' = ln(1-`uRatio')					if 	$ML_y1==1	& `ra'==0
        replace `lnf' = (ln(1-`uRatio')/2)+(ln(`uRatio')/2)		if	$ML_y1==-1	& `ra'==0

    }

end

* Exponential model with CRRA version with smooth derivatives at r=1
program MLex_ra_crra2_lam

    args lnf r LNdelta muRA muIDR

    tempvar ra t mA mB prob1 prob2 mA1 mA2 mB1 mB2 yA1 yA2 yB1 yB2 wp1 wp2 euA euB euRatio y0 y1 
    tempvar tau mA mB uA uB uRatio baseSR baseMR rMR scale delta lam0 lam1 lambda1 lambda2
    tempvar task1 task2 task3 task4 task5 task6

    * Define temporary variables
	forvalues X = 1(1)$lambda0 {
	    tempvar uA_S_`X' uB_S_`X'
	}
	forvalues X = 1(1)$lambda1 {
	    tempvar uA_L1_`X' uB_L1_`X'
	}
	forvalues X = 1(1)$lambda2 {
	    tempvar uA_L2_`X' uB_L2_`X'
	}
	forvalues X = 1(1)$lambda3 {
	    tempvar uA_L3_`X' uB_L3_`X'
	}
	forvalues X = 1(1)$lambda4 {
	    tempvar uA_L4_`X' uB_L4_`X'
	}
	forvalues X = 1(1)$lambda5 {
	    tempvar uA_L5_`X' uB_L5_`X'
	}
	forvalues X = 1(1)$lambda6 {
	    tempvar uA_L6_`X' uB_L6_`X'
	}

	syntax varlist [if] [in] [fweight pweight] [, noLOg	Robust CLuster(varname)	svy	EForm *]
	marksample touse
	if "`svy'" != "" {
		svymarkout `touse'
	}
 
    quietly {

        * initialize horizon and payoffs
        generate double `ra' 	= $ML_y6
        generate double `t' 	= $ML_y2/360
        generate double `tau' 	= (30+$ML_y2)/360
        generate double `mA'	= $ML_y3
        generate double `mB'	= $ML_y4	
        generate double `prob1' = $ML_y5      
        generate double `prob2' = 1 - `prob1'
        generate double `mA1' 	= $ML_y7      
        generate double `mA2' 	= $ML_y8
        generate double `mB1' 	= $ML_y9
        generate double `mB2' 	= $ML_y10
	generate double `task1' = $ML_y11
	generate double `task2' = $ML_y12
	generate double `task3' = $ML_y13
	generate double `task4' = $ML_y14
	generate double `task5' = $ML_y15
	generate double `task6' = $ML_y16

        * transform parameters
        generate double `delta' = exp(`LNdelta')

        * estimating over risk responses
        generate double `yA1' = (($omega+`mA1')^(1-`r')-1)/(1-`r')	if `ra'==1
        generate double `yA2' = (($omega+`mA2')^(1-`r')-1)/(1-`r')	if `ra'==1
        generate double `yB1' = (($omega+`mB1')^(1-`r')-1)/(1-`r')	if `ra'==1
        generate double `yB2' = (($omega+`mB2')^(1-`r')-1)/(1-`r')	if `ra'==1

        generate double `wp1' = `prob1'					if `ra'==1	
        generate double `wp2' = `prob2'					if `ra'==1
        generate double `euA' = (`wp1'*`yA1')+(`wp2'*`yA2')		if `ra'==1      
        generate double `euB' = (`wp1'*`yB1')+(`wp2'*`yB2')		if `ra'==1

        generate double `euRatio' = (`euA'^(1/`muRA'))/((`euA'^(1/`muRA'))+(`euB'^(1/`muRA')))	if `ra'==1

        * save predicted RA results
        replace Pchoices=0 if `euA'>`euB' & `ra'==1
        replace Pchoices=1 if `euA'<`euB' & `ra'==1

        replace `lnf' = ln(`euRatio')					if $ML_y1==0	& `ra'==1
        replace `lnf' = ln(1-`euRatio')					if $ML_y1==1	& `ra'==1
        replace `lnf' = (ln(1-`euRatio')/2)+(ln(`euRatio')/2)		if $ML_y1==-1	& `ra'==1

        * estimating over time responses
        generate double `y0' = 0
        generate double `y1' = 0

	* Exponential discounting
	forvalues X = 1(1)$lambda0 {
	    generate double `uA_S_`X'' = (1 / ((1+`delta')^(30/360 + (`X'-1)/360 )) ) * (($omega + `mA'/$lambda0)^(1-`r')-1)/(1-`r')	if `ra'==0
	    generate double `uB_S_`X'' = (1 / ((1+`delta')^(30/360 + (`X'-1)/360 )) ) * (($omega                )^(1-`r')-1)/(1-`r')	if `ra'==0

	    replace `y0' = `y0' +  `uA_S_`X''		if `ra'==0
	    replace `y1' = `y1' +  `uB_S_`X''		if `ra'==0
	}	

	forvalues X = 1(1)$lambda1 {
	    generate double `uA_L1_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r')-1)/(1-`r')	if `ra'==0 & `task1'==1
	    generate double `uB_L1_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda1)^(1-`r')-1)/(1-`r')	if `ra'==0 & `task1'==1

	    replace `y0' = `y0'  +  `uA_L1_`X''		if `ra'==0 & `task1'==1
	    replace `y1' = `y1'  +  `uB_L1_`X''		if `ra'==0 & `task1'==1
	}	

	forvalues X = 1(1)$lambda2 {
	    generate double `uA_L2_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r')-1)/(1-`r')	if `ra'==0 & `task2'==1
	    generate double `uB_L2_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda2)^(1-`r')-1)/(1-`r')	if `ra'==0 & `task2'==1

	    replace `y0' = `y0'  +  `uA_L2_`X''		if `ra'==0 & `task2'==1
	    replace `y1' = `y1'  +  `uB_L2_`X''		if `ra'==0 & `task2'==1
	}	

	forvalues X = 1(1)$lambda3 {
	    generate double `uA_L3_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r')-1)/(1-`r')	if `ra'==0 & `task3'==1
	    generate double `uB_L3_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda3)^(1-`r')-1)/(1-`r')	if `ra'==0 & `task3'==1

	    replace `y0' = `y0'  +  `uA_L3_`X''		if `ra'==0 & `task3'==1
	    replace `y1' = `y1'  +  `uB_L3_`X''		if `ra'==0 & `task3'==1
	}	

	forvalues X = 1(1)$lambda4 {
	    generate double `uA_L4_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r')-1)/(1-`r')	if `ra'==0 & `task4'==1
	    generate double `uB_L4_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda4)^(1-`r')-1)/(1-`r')	if `ra'==0 & `task4'==1

	    replace `y0' = `y0'  +  `uA_L4_`X''		if `ra'==0 & `task4'==1
	    replace `y1' = `y1'  +  `uB_L4_`X''		if `ra'==0 & `task4'==1
	}	

	forvalues X = 1(1)$lambda5 {
	    generate double `uA_L5_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r')-1)/(1-`r')	if `ra'==0 & `task5'==1
	    generate double `uB_L5_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda5)^(1-`r')-1)/(1-`r')	if `ra'==0 & `task5'==1

	    replace `y0' = `y0'  +  `uA_L5_`X''		if `ra'==0 & `task5'==1
	    replace `y1' = `y1'  +  `uB_L5_`X''		if `ra'==0 & `task5'==1
	}	

	forvalues X = 1(1)$lambda6 {
	    generate double `uA_L6_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r')-1)/(1-`r')	if `ra'==0 & `task6'==1
	    generate double `uB_L6_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda6)^(1-`r')-1)/(1-`r')	if `ra'==0 & `task6'==1

	    replace `y0' = `y0'  +  `uA_L6_`X''		if `ra'==0 & `task6'==1
	    replace `y1' = `y1'  +  `uB_L6_`X''		if `ra'==0 & `task6'==1
	}	

        * save DR results
        replace Pchoices=0 if `y0'>`y1' & `ra'==0
        replace Pchoices=1 if `y0'<`y1' & `ra'==0

        generate double `uRatio' = (`y0'^(1/`muIDR'))/((`y0'^(1/`muIDR'))+(`y1'^(1/`muIDR'))) 	if `ra'==0

        replace `lnf' = ln(`uRatio')					if 	$ML_y1==0	& `ra'==0
        replace `lnf' = ln(1-`uRatio')					if 	$ML_y1==1	& `ra'==0
        replace `lnf' = (ln(1-`uRatio')/2)+(ln(`uRatio')/2)		if	$ML_y1==-1	& `ra'==0

    }

end

* Exponential model with CRRA, with delta defined as delta and not LNdelta
program MLex_ra_crra_lam_

    args lnf r delta muRA muIDR

    tempvar ra t mA mB prob1 prob2 mA1 mA2 mB1 mB2 yA1 yA2 yB1 yB2 wp1 wp2 euA euB euRatio y0 y1 
    tempvar tau mA mB uA uB uRatio baseSR baseMR rMR scale lam0 lam1 lambda1 lambda2
    tempvar task1 task2 task3 task4 task5 task6

    * Define temporary variables
	forvalues X = 1(1)$lambda0 {
	    tempvar uA_S_`X' uB_S_`X'
	}
	forvalues X = 1(1)$lambda1 {
	    tempvar uA_L1_`X' uB_L1_`X'
	}
	forvalues X = 1(1)$lambda2 {
	    tempvar uA_L2_`X' uB_L2_`X'
	}
	forvalues X = 1(1)$lambda3 {
	    tempvar uA_L3_`X' uB_L3_`X'
	}
	forvalues X = 1(1)$lambda4 {
	    tempvar uA_L4_`X' uB_L4_`X'
	}
	forvalues X = 1(1)$lambda5 {
	    tempvar uA_L5_`X' uB_L5_`X'
	}
	forvalues X = 1(1)$lambda6 {
	    tempvar uA_L6_`X' uB_L6_`X'
	}

	syntax varlist [if] [in] [fweight pweight] [, noLOg	Robust CLuster(varname)	svy	EForm *]
	marksample touse
	if "`svy'" != "" {
		svymarkout `touse'
	}
 
    quietly {

        * initialize horizon and payoffs
        generate double `ra' 	= $ML_y6
        generate double `t' 	= $ML_y2/360
        generate double `tau' 	= (30+$ML_y2)/360
        generate double `mA'	= $ML_y3
        generate double `mB'	= $ML_y4	
        generate double `prob1' = $ML_y5      
        generate double `prob2' = 1 - `prob1'
        generate double `mA1' 	= $ML_y7      
        generate double `mA2' 	= $ML_y8
        generate double `mB1' 	= $ML_y9
        generate double `mB2' 	= $ML_y10
	generate double `task1' = $ML_y11
	generate double `task2' = $ML_y12
	generate double `task3' = $ML_y13
	generate double `task4' = $ML_y14
	generate double `task5' = $ML_y15
	generate double `task6' = $ML_y16

        * estimating over risk responses
        generate double `yA1' = (($omega+`mA1')^(1-`r'))/(1-`r')	if `ra'==1
        generate double `yA2' = (($omega+`mA2')^(1-`r'))/(1-`r')	if `ra'==1
        generate double `yB1' = (($omega+`mB1')^(1-`r'))/(1-`r')	if `ra'==1
        generate double `yB2' = (($omega+`mB2')^(1-`r'))/(1-`r')	if `ra'==1

        generate double `wp1' = `prob1'					if `ra'==1	
        generate double `wp2' = `prob2'					if `ra'==1
        generate double `euA' = (`wp1'*`yA1')+(`wp2'*`yA2')		if `ra'==1      
        generate double `euB' = (`wp1'*`yB1')+(`wp2'*`yB2')		if `ra'==1

        generate double `euRatio' = (`euA'^(1/`muRA'))/((`euA'^(1/`muRA'))+(`euB'^(1/`muRA')))	if `ra'==1

        replace `lnf' = ln(`euRatio')					if $ML_y1==0	& `ra'==1
        replace `lnf' = ln(1-`euRatio')					if $ML_y1==1	& `ra'==1
        replace `lnf' = (ln(1-`euRatio')/2)+(ln(`euRatio')/2)		if $ML_y1==-1	& `ra'==1

        * estimating over time responses
        generate double `y0' = 0
        generate double `y1' = 0

	* Exponential discounting
	forvalues X = 1(1)$lambda0 {
	    generate double `uA_S_`X'' = (1 / ((1+`delta')^(30/360 + (`X'-1)/360 )) ) * (($omega + `mA'/$lambda0)^(1-`r'))/(1-`r')	if `ra'==0
	    generate double `uB_S_`X'' = (1 / ((1+`delta')^(30/360 + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0

	    replace `y0' = `y0' +  `uA_S_`X''		if `ra'==0
	    replace `y1' = `y1' +  `uB_S_`X''		if `ra'==0
	}	

	forvalues X = 1(1)$lambda1 {
	    generate double `uA_L1_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task1'==1
	    generate double `uB_L1_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda1)^(1-`r'))/(1-`r')	if `ra'==0 & `task1'==1

	    replace `y0' = `y0'  +  `uA_L1_`X''		if `ra'==0 & `task1'==1
	    replace `y1' = `y1'  +  `uB_L1_`X''		if `ra'==0 & `task1'==1
	}	

	forvalues X = 1(1)$lambda2 {
	    generate double `uA_L2_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task2'==1
	    generate double `uB_L2_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda2)^(1-`r'))/(1-`r')	if `ra'==0 & `task2'==1

	    replace `y0' = `y0'  +  `uA_L2_`X''		if `ra'==0 & `task2'==1
	    replace `y1' = `y1'  +  `uB_L2_`X''		if `ra'==0 & `task2'==1
	}	

	forvalues X = 1(1)$lambda3 {
	    generate double `uA_L3_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task3'==1
	    generate double `uB_L3_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda3)^(1-`r'))/(1-`r')	if `ra'==0 & `task3'==1

	    replace `y0' = `y0'  +  `uA_L3_`X''		if `ra'==0 & `task3'==1
	    replace `y1' = `y1'  +  `uB_L3_`X''		if `ra'==0 & `task3'==1
	}	

	forvalues X = 1(1)$lambda4 {
	    generate double `uA_L4_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task4'==1
	    generate double `uB_L4_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda4)^(1-`r'))/(1-`r')	if `ra'==0 & `task4'==1

	    replace `y0' = `y0'  +  `uA_L4_`X''		if `ra'==0 & `task4'==1
	    replace `y1' = `y1'  +  `uB_L4_`X''		if `ra'==0 & `task4'==1
	}	

	forvalues X = 1(1)$lambda5 {
	    generate double `uA_L5_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task5'==1
	    generate double `uB_L5_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda5)^(1-`r'))/(1-`r')	if `ra'==0 & `task5'==1

	    replace `y0' = `y0'  +  `uA_L5_`X''		if `ra'==0 & `task5'==1
	    replace `y1' = `y1'  +  `uB_L5_`X''		if `ra'==0 & `task5'==1
	}	

	forvalues X = 1(1)$lambda6 {
	    generate double `uA_L6_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task6'==1
	    generate double `uB_L6_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda6)^(1-`r'))/(1-`r')	if `ra'==0 & `task6'==1

	    replace `y0' = `y0'  +  `uA_L6_`X''		if `ra'==0 & `task6'==1
	    replace `y1' = `y1'  +  `uB_L6_`X''		if `ra'==0 & `task6'==1
	}	

        generate double `uRatio' = (`y0'^(1/`muIDR'))/((`y0'^(1/`muIDR'))+(`y1'^(1/`muIDR'))) 	if `ra'==0

        replace `lnf' = ln(`uRatio')					if 	$ML_y1==0	& `ra'==0
        replace `lnf' = ln(1-`uRatio')					if 	$ML_y1==1	& `ra'==0
        replace `lnf' = (ln(1-`uRatio')/2)+(ln(`uRatio')/2)		if	$ML_y1==-1	& `ra'==0

    }

end


* Mazur-hyperbolic model with CRRA
program MLh_crra_lam

    args lnf r LNalphaD muRA muIDR

    tempvar ra t mA mB prob1 prob2 mA1 mA2 mB1 mB2 yA1 yA2 yB1 yB2 wp1 wp2 euA euB euRatio y0 y1 
    tempvar tau mA mB uA uB uRatio baseSR baseMR rMR scale alphaD lam0 lam1 lambda1 lambda2
    tempvar task1 task2 task3 task4 task5 task6

    * Define temporary variables
	forvalues X = 1(1)$lambda0 {
	    tempvar uA_S_`X' uB_S_`X'
	}
	forvalues X = 1(1)$lambda1 {
	    tempvar uA_L1_`X' uB_L1_`X'
	}
	forvalues X = 1(1)$lambda2 {
	    tempvar uA_L2_`X' uB_L2_`X'
	}
	forvalues X = 1(1)$lambda3 {
	    tempvar uA_L3_`X' uB_L3_`X'
	}
	forvalues X = 1(1)$lambda4 {
	    tempvar uA_L4_`X' uB_L4_`X'
	}
	forvalues X = 1(1)$lambda5 {
	    tempvar uA_L5_`X' uB_L5_`X'
	}
	forvalues X = 1(1)$lambda6 {
	    tempvar uA_L6_`X' uB_L6_`X'
	}

	syntax varlist [if] [in] [fweight pweight] [, noLOg	Robust CLuster(varname)	svy	EForm *]
	marksample touse
	if "`svy'" != "" {
		svymarkout `touse'
	}
 
    quietly {

        * initialize horizon and payoffs
        generate double `ra' 	= $ML_y6
        generate double `t' 	= $ML_y2/360
        generate double `tau' 	= (30+$ML_y2)/360
        generate double `mA'	= $ML_y3
        generate double `mB'	= $ML_y4	
        generate double `prob1' = $ML_y5      
        generate double `prob2' = 1 - `prob1'
        generate double `mA1' 	= $ML_y7      
        generate double `mA2' 	= $ML_y8
        generate double `mB1' 	= $ML_y9
        generate double `mB2' 	= $ML_y10
	generate double `task1' = $ML_y11
	generate double `task2' = $ML_y12
	generate double `task3' = $ML_y13
	generate double `task4' = $ML_y14
	generate double `task5' = $ML_y15
	generate double `task6' = $ML_y16

        * transform parameters
        generate double `alphaD' = exp(`LNalphaD')

        * estimating over risk responses
        generate double `yA1' = (($omega+`mA1')^(1-`r'))/(1-`r')	if `ra'==1
        generate double `yA2' = (($omega+`mA2')^(1-`r'))/(1-`r')	if `ra'==1
        generate double `yB1' = (($omega+`mB1')^(1-`r'))/(1-`r')	if `ra'==1
        generate double `yB2' = (($omega+`mB2')^(1-`r'))/(1-`r')	if `ra'==1

        generate double `wp1' = `prob1'					if `ra'==1	
        generate double `wp2' = `prob2'					if `ra'==1
        generate double `euA' = (`wp1'*`yA1')+(`wp2'*`yA2')		if `ra'==1      
        generate double `euB' = (`wp1'*`yB1')+(`wp2'*`yB2')		if `ra'==1

        generate double `euRatio' = (`euA'^(1/`muRA'))/((`euA'^(1/`muRA'))+(`euB'^(1/`muRA')))	if `ra'==1

        replace `lnf' = ln(`euRatio')					if $ML_y1==0	& `ra'==1
        replace `lnf' = ln(1-`euRatio')					if $ML_y1==1	& `ra'==1
        replace `lnf' = (ln(1-`euRatio')/2)+(ln(`euRatio')/2)		if $ML_y1==-1	& `ra'==1

        * estimating over time responses
        generate double `y0' = 0
        generate double `y1' = 0

	* Mazur-hyperbolic discounting
	forvalues X = 1(1)$lambda0 {
	    generate double `uA_S_`X'' = (1 / (1+`alphaD'*(30/360 + (`X'-1)/360)) ) * (($omega + `mA'/$lambda0)^(1-`r'))/(1-`r')	if `ra'==0
	    generate double `uB_S_`X'' = (1 / (1+`alphaD'*(30/360 + (`X'-1)/360)) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0

	    replace `y0' = `y0' +  `uA_S_`X''		if `ra'==0
	    replace `y1' = `y1' +  `uB_S_`X''		if `ra'==0
	}	

	forvalues X = 1(1)$lambda1 {
	    generate double `uA_L1_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task1'==1
	    generate double `uB_L1_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda1)^(1-`r'))/(1-`r')	if `ra'==0 & `task1'==1

	    replace `y0' = `y0'  +  `uA_L1_`X''		if `ra'==0 & `task1'==1
	    replace `y1' = `y1'  +  `uB_L1_`X''		if `ra'==0 & `task1'==1
	}	

	forvalues X = 1(1)$lambda2 {
	    generate double `uA_L2_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task2'==1
	    generate double `uB_L2_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda2)^(1-`r'))/(1-`r')	if `ra'==0 & `task2'==1

	    replace `y0' = `y0'  +  `uA_L2_`X''		if `ra'==0 & `task2'==1
	    replace `y1' = `y1'  +  `uB_L2_`X''		if `ra'==0 & `task2'==1
	}	

	forvalues X = 1(1)$lambda3 {
	    generate double `uA_L3_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task3'==1
	    generate double `uB_L3_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda3)^(1-`r'))/(1-`r')	if `ra'==0 & `task3'==1

	    replace `y0' = `y0'  +  `uA_L3_`X''		if `ra'==0 & `task3'==1
	    replace `y1' = `y1'  +  `uB_L3_`X''		if `ra'==0 & `task3'==1
	}	

	forvalues X = 1(1)$lambda4 {
	    generate double `uA_L4_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task4'==1
	    generate double `uB_L4_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda4)^(1-`r'))/(1-`r')	if `ra'==0 & `task4'==1

	    replace `y0' = `y0'  +  `uA_L4_`X''		if `ra'==0 & `task4'==1
	    replace `y1' = `y1'  +  `uB_L4_`X''		if `ra'==0 & `task4'==1
	}	

	forvalues X = 1(1)$lambda5 {
	    generate double `uA_L5_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task5'==1
	    generate double `uB_L5_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda5)^(1-`r'))/(1-`r')	if `ra'==0 & `task5'==1

	    replace `y0' = `y0'  +  `uA_L5_`X''		if `ra'==0 & `task5'==1
	    replace `y1' = `y1'  +  `uB_L5_`X''		if `ra'==0 & `task5'==1
	}	

	forvalues X = 1(1)$lambda6 {
	    generate double `uA_L6_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task6'==1
	    generate double `uB_L6_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda6)^(1-`r'))/(1-`r')	if `ra'==0 & `task6'==1

	    replace `y0' = `y0'  +  `uA_L6_`X''		if `ra'==0 & `task6'==1
	    replace `y1' = `y1'  +  `uB_L6_`X''		if `ra'==0 & `task6'==1
	}	

        generate double `uRatio' = (`y0'^(1/`muIDR'))/((`y0'^(1/`muIDR'))+(`y1'^(1/`muIDR'))) 	if `ra'==0

        replace `lnf' = ln(`uRatio')					if 	$ML_y1==0	& `ra'==0
        replace `lnf' = ln(1-`uRatio')					if 	$ML_y1==1	& `ra'==0
        replace `lnf' = (ln(1-`uRatio')/2)+(ln(`uRatio')/2)		if	$ML_y1==-1	& `ra'==0

    }

end


* Prelec-hyperbolic model with CRRA
program MLp_crra_lam

    args lnf r betaD alphaD_ muRA muIDR

    tempvar ra t mA mB prob1 prob2 mA1 mA2 mB1 mB2 yA1 yA2 yB1 yB2 wp1 wp2 euA euB euRatio y0 y1 
    tempvar tau mA mB uA uB uRatio baseSR baseMR rMR scale lam0 lam1 lambda1 lambda2
    tempvar task1 task2 task3 task4 task5 task6

    * Define temporary variables
	forvalues X = 1(1)$lambda0 {
	    tempvar uA_S_`X' uB_S_`X'
	}
	forvalues X = 1(1)$lambda1 {
	    tempvar uA_L1_`X' uB_L1_`X'
	}
	forvalues X = 1(1)$lambda2 {
	    tempvar uA_L2_`X' uB_L2_`X'
	}
	forvalues X = 1(1)$lambda3 {
	    tempvar uA_L3_`X' uB_L3_`X'
	}
	forvalues X = 1(1)$lambda4 {
	    tempvar uA_L4_`X' uB_L4_`X'
	}
	forvalues X = 1(1)$lambda5 {
	    tempvar uA_L5_`X' uB_L5_`X'
	}
	forvalues X = 1(1)$lambda6 {
	    tempvar uA_L6_`X' uB_L6_`X'
	}

	syntax varlist [if] [in] [fweight pweight] [, noLOg	Robust CLuster(varname)	svy	EForm *]
	marksample touse
	if "`svy'" != "" {
		svymarkout `touse'
	}
 
    quietly {

        * initialize horizon and payoffs
        generate double `ra' 	= $ML_y6
        generate double `t' 	= $ML_y2/360
        generate double `tau' 	= (30+$ML_y2)/360
        generate double `mA'	= $ML_y3
        generate double `mB'	= $ML_y4	
        generate double `prob1' = $ML_y5      
        generate double `prob2' = 1 - `prob1'
        generate double `mA1' 	= $ML_y7      
        generate double `mA2' 	= $ML_y8
        generate double `mB1' 	= $ML_y9
        generate double `mB2' 	= $ML_y10
	generate double `task1' = $ML_y11
	generate double `task2' = $ML_y12
	generate double `task3' = $ML_y13
	generate double `task4' = $ML_y14
	generate double `task5' = $ML_y15
	generate double `task6' = $ML_y16

        * estimating over risk responses
        generate double `yA1' = (($omega+`mA1')^(1-`r'))/(1-`r')	if `ra'==1
        generate double `yA2' = (($omega+`mA2')^(1-`r'))/(1-`r')	if `ra'==1
        generate double `yB1' = (($omega+`mB1')^(1-`r'))/(1-`r')	if `ra'==1
        generate double `yB2' = (($omega+`mB2')^(1-`r'))/(1-`r')	if `ra'==1

        generate double `wp1' = `prob1'					if `ra'==1	
        generate double `wp2' = `prob2'					if `ra'==1
        generate double `euA' = (`wp1'*`yA1')+(`wp2'*`yA2')		if `ra'==1      
        generate double `euB' = (`wp1'*`yB1')+(`wp2'*`yB2')		if `ra'==1

        generate double `euRatio' = (`euA'^(1/`muRA'))/((`euA'^(1/`muRA'))+(`euB'^(1/`muRA')))	if `ra'==1

        replace `lnf' = ln(`euRatio')					if $ML_y1==0	& `ra'==1
        replace `lnf' = ln(1-`euRatio')					if $ML_y1==1	& `ra'==1
        replace `lnf' = (ln(1-`euRatio')/2)+(ln(`euRatio')/2)		if $ML_y1==-1	& `ra'==1

        * estimating over time responses
        generate double `y0' = 0
        generate double `y1' = 0

	forvalues X = 1(1)$lambda0 {
	    generate double `uA_S_`X'' = exp(-`betaD'*(30/360 + (`X'-1)/360)^(`alphaD_')) * (($omega + `mA'/$lambda0)^(1-`r'))/(1-`r')	if `ra'==0
	    generate double `uB_S_`X'' = exp(-`betaD'*(30/360 + (`X'-1)/360)^(`alphaD_')) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0

	    replace `y0' = `y0' +  `uA_S_`X''		if `ra'==0
	    replace `y1' = `y1' +  `uB_S_`X''		if `ra'==0
	}	

	forvalues X = 1(1)$lambda1 {
	    generate double `uA_L1_`X'' = exp(-`betaD'*(`tau' + (`X'-1)/360)^(`alphaD_')) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task1'==1
	    generate double `uB_L1_`X'' = exp(-`betaD'*(`tau' + (`X'-1)/360)^(`alphaD_')) * (($omega + `mB'/$lambda1)^(1-`r'))/(1-`r')	if `ra'==0 & `task1'==1

	    replace `y0' = `y0'  +  `uA_L1_`X''		if `ra'==0 & `task1'==1
	    replace `y1' = `y1'  +  `uB_L1_`X''		if `ra'==0 & `task1'==1
	}	

	forvalues X = 1(1)$lambda2 {
	    generate double `uA_L2_`X'' = exp(-`betaD'*(`tau' + (`X'-1)/360)^(`alphaD_')) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task2'==1
	    generate double `uB_L2_`X'' = exp(-`betaD'*(`tau' + (`X'-1)/360)^(`alphaD_')) * (($omega + `mB'/$lambda2)^(1-`r'))/(1-`r')	if `ra'==0 & `task2'==1

	    replace `y0' = `y0'  +  `uA_L2_`X''		if `ra'==0 & `task2'==1
	    replace `y1' = `y1'  +  `uB_L2_`X''		if `ra'==0 & `task2'==1
	}	

	forvalues X = 1(1)$lambda3 {
	    generate double `uA_L3_`X'' = exp(-`betaD'*(`tau' + (`X'-1)/360)^(`alphaD_')) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task3'==1
	    generate double `uB_L3_`X'' = exp(-`betaD'*(`tau' + (`X'-1)/360)^(`alphaD_')) * (($omega + `mB'/$lambda3)^(1-`r'))/(1-`r')	if `ra'==0 & `task3'==1

	    replace `y0' = `y0'  +  `uA_L3_`X''		if `ra'==0 & `task3'==1
	    replace `y1' = `y1'  +  `uB_L3_`X''		if `ra'==0 & `task3'==1
	}	

	forvalues X = 1(1)$lambda4 {
	    generate double `uA_L4_`X'' = exp(-`betaD'*(`tau' + (`X'-1)/360)^(`alphaD_')) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task4'==1
	    generate double `uB_L4_`X'' = exp(-`betaD'*(`tau' + (`X'-1)/360)^(`alphaD_')) * (($omega + `mB'/$lambda4)^(1-`r'))/(1-`r')	if `ra'==0 & `task4'==1

	    replace `y0' = `y0'  +  `uA_L4_`X''		if `ra'==0 & `task4'==1
	    replace `y1' = `y1'  +  `uB_L4_`X''		if `ra'==0 & `task4'==1
	}	

	forvalues X = 1(1)$lambda5 {
	    generate double `uA_L5_`X'' = exp(-`betaD'*(`tau' + (`X'-1)/360)^(`alphaD_')) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task5'==1
	    generate double `uB_L5_`X'' = exp(-`betaD'*(`tau' + (`X'-1)/360)^(`alphaD_')) * (($omega + `mB'/$lambda5)^(1-`r'))/(1-`r')	if `ra'==0 & `task5'==1

	    replace `y0' = `y0'  +  `uA_L5_`X''		if `ra'==0 & `task5'==1
	    replace `y1' = `y1'  +  `uB_L5_`X''		if `ra'==0 & `task5'==1
	}	

	forvalues X = 1(1)$lambda6 {
	    generate double `uA_L6_`X'' = exp(-`betaD'*(`tau' + (`X'-1)/360)^(`alphaD_')) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task6'==1
	    generate double `uB_L6_`X'' = exp(-`betaD'*(`tau' + (`X'-1)/360)^(`alphaD_')) * (($omega + `mB'/$lambda6)^(1-`r'))/(1-`r')	if `ra'==0 & `task6'==1

	    replace `y0' = `y0'  +  `uA_L6_`X''		if `ra'==0 & `task6'==1
	    replace `y1' = `y1'  +  `uB_L6_`X''		if `ra'==0 & `task6'==1
	}	

        generate double `uRatio' = (`y0'^(1/`muIDR'))/((`y0'^(1/`muIDR'))+(`y1'^(1/`muIDR'))) 	if `ra'==0

        replace `lnf' = ln(`uRatio')					if 	$ML_y1==0	& `ra'==0
        replace `lnf' = ln(1-`uRatio')					if 	$ML_y1==1	& `ra'==0
        replace `lnf' = (ln(1-`uRatio')/2)+(ln(`uRatio')/2)		if	$ML_y1==-1	& `ra'==0

    }

end


** MIXTURE MODELS **
* Mixture model of exponential and Mazur-hyperbolic discounting with risk neutrality
program MLmix_ex_h_rn_lam

    args lnf LNdelta LNalphaD kappa muIDR

    tempvar t mA mB y0 y1 tau uRatio delta alphaD lnfEX lnfPH f1 f2 p1 p2
    tempvar task1 task2 task3 task4 task5 task6

    * Define temporary variables
	forvalues X = 1(1)$lambda0 {
	    tempvar uA_S_`X'
	}
	forvalues X = 1(1)$lambda1 {
	    tempvar uB_L1_`X'
	}
	forvalues X = 1(1)$lambda2 {
	    tempvar uB_L2_`X'
	}
	forvalues X = 1(1)$lambda3 {
	    tempvar uB_L3_`X'
	}
	forvalues X = 1(1)$lambda4 {
	    tempvar uB_L4_`X'
	}
	forvalues X = 1(1)$lambda5 {
	    tempvar uB_L5_`X'
	}
	forvalues X = 1(1)$lambda6 {
	    tempvar uB_L6_`X'
	}

	syntax varlist [if] [in] [fweight pweight] [, noLOg	Robust CLuster(varname)	svy	EForm *]
	marksample touse
	if "`svy'" != "" {
		svymarkout `touse'
	}
 
    quietly {

        * initialize horizon and payoffs
        generate double `t' 	= $ML_y2/360
        generate double `tau' 	= (30+$ML_y2)/360
        generate double `mA'	= $ML_y3
        generate double `mB'	= $ML_y4	
	generate double `task1' = $ML_y11
	generate double `task2' = $ML_y12
	generate double `task3' = $ML_y13
	generate double `task4' = $ML_y14
	generate double `task5' = $ML_y15
	generate double `task6' = $ML_y16

        * transform parameters
        generate double `delta'  = exp(`LNdelta')
        generate double `alphaD' = exp(`LNalphaD')

        * estimating over time responses
        generate double `y0' = 0
        generate double `y1' = 0

	* Exponential discounting
	forvalues X = 1(1)$lambda0 {
	    generate double `uA_S_`X'' = (1 / ((1+`delta')^(30/360 + (`X'-1)/360 )) ) * (`mA'/$lambda0)

	    replace `y0' = `y0' + `uA_S_`X''
	}	

	forvalues X = 1(1)$lambda1 {
	    generate double `uB_L1_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (`mB'/$lambda1)		if `task1'==1

	    replace `y1' = `y1'  +  `uB_L1_`X''		if `task1'==1
	}	

	forvalues X = 1(1)$lambda2 {
	    generate double `uB_L2_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (`mB'/$lambda2)		if `task2'==1

	    replace `y1' = `y1'  +  `uB_L2_`X''		if `task2'==1
	}	

	forvalues X = 1(1)$lambda3 {
	    generate double `uB_L3_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (`mB'/$lambda3)		if `task3'==1

	    replace `y1' = `y1'  +  `uB_L3_`X''		if `task3'==1
	}	

	forvalues X = 1(1)$lambda4 {
	    generate double `uB_L4_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (`mB'/$lambda4)		if `task4'==1

	    replace `y1' = `y1'  +  `uB_L4_`X''		if `task4'==1
	}	

	forvalues X = 1(1)$lambda5 {
	    generate double `uB_L5_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (`mB'/$lambda5)		if `task5'==1

	    replace `y1' = `y1'  +  `uB_L5_`X''		if `task5'==1
	}	

	forvalues X = 1(1)$lambda6 {
	    generate double `uB_L6_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (`mB'/$lambda6)		if `task6'==1

	    replace `y1' = `y1'  +  `uB_L6_`X''		if `task6'==1
	}	

        generate double `uRatio' = (`y0'^(1/`muIDR'))/((`y0'^(1/`muIDR'))+(`y1'^(1/`muIDR')))

        generate double `lnfEX' = ln(`uRatio')					if 	$ML_y1==0
        replace         `lnfEX' = ln(1-`uRatio')				if 	$ML_y1==1
        replace         `lnfEX' = (ln(1-`uRatio')/2)+(ln(`uRatio')/2)		if	$ML_y1==-1

	* Mazur-hyperbolic discounting
	forvalues X = 1(1)$lambda0 {
	    replace `uA_S_`X''  = (1 / (1+`alphaD'*(30/360 + (`X'-1)/360)) ) * (`mA'/$lambda0)

	    replace `y0' = `y0' + `uA_S_`X''
	}	

	forvalues X = 1(1)$lambda1 {
	    replace `uB_L1_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360)) ) * (`mB'/$lambda1)		if `task1'==1

	    replace `y1' = `y1'  +  `uB_L1_`X''		if `task1'==1
	}	

	forvalues X = 1(1)$lambda2 {
	    replace `uB_L2_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360)) ) * (`mB'/$lambda2)		if `task2'==1

	    replace `y1' = `y1'  +  `uB_L2_`X''		if `task2'==1
	}	

	forvalues X = 1(1)$lambda3 {
	    replace `uB_L3_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360)) ) * (`mB'/$lambda3)		if `task3'==1

	    replace `y1' = `y1'  +  `uB_L3_`X''		if `task3'==1
	}	

	forvalues X = 1(1)$lambda4 {
	    replace `uB_L4_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360)) ) * (`mB'/$lambda4)		if `task4'==1

	    replace `y1' = `y1'  +  `uB_L4_`X''		if `task4'==1
	}	

	forvalues X = 1(1)$lambda5 {
	    replace `uB_L5_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360)) ) * (`mB'/$lambda5)		if `task5'==1

	    replace `y1' = `y1'  +  `uB_L5_`X''		if `task5'==1
	}	

	forvalues X = 1(1)$lambda6 {
	    replace `uB_L6_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360)) ) * (`mB'/$lambda6)		if `task6'==1

	    replace `y1' = `y1'  +  `uB_L6_`X''		if `task6'==1
	}	

        replace `uRatio' = (`y0'^(1/`muIDR'))/((`y0'^(1/`muIDR'))+(`y1'^(1/`muIDR')))

        generate double `lnfPH' = ln(`uRatio')					if 	$ML_y1==0
        replace         `lnfPH' = ln(1-`uRatio')				if 	$ML_y1==1
        replace         `lnfPH' = (ln(1-`uRatio')/2)+(ln(`uRatio')/2)		if	$ML_y1==-1

        * get the grand index for the discounting choices
        generate double `f1' = exp(`lnfEX')
        generate double `f2' = exp(`lnfPH')
        generate double `p1' =            1/(1+exp(`kappa'))
        generate double `p2' = exp(`kappa')/(1+exp(`kappa'))
        replace	`lnf' = ln((`p1' * `f1') + (`p2' * `f2'))

    }

end

* Mixture model of exponential and Mazur-hyperbolic discounting with CRRA
program MLmix_ex_h_lam

    args lnf r LNdelta LNalphaD kappa muRA muIDR

    tempvar ra t mA mB prob1 prob2 mA1 mA2 mB1 mB2 yA1 yA2 yB1 yB2 wp1 wp2 euA euB euRatio y0 y1 
    tempvar tau mA mB uA uB uRatio baseSR baseMR rMR scale delta alphaD lnfEX lnfPH f1 f2 p1 p2
    tempvar task1 task2 task3 task4 task5 task6

    * Define temporary variables
	forvalues X = 1(1)$lambda0 {
	    tempvar uA_S_`X' uB_S_`X'
	}
	forvalues X = 1(1)$lambda1 {
	    tempvar uA_L1_`X' uB_L1_`X'
	}
	forvalues X = 1(1)$lambda2 {
	    tempvar uA_L2_`X' uB_L2_`X'
	}
	forvalues X = 1(1)$lambda3 {
	    tempvar uA_L3_`X' uB_L3_`X'
	}
	forvalues X = 1(1)$lambda4 {
	    tempvar uA_L4_`X' uB_L4_`X'
	}
	forvalues X = 1(1)$lambda5 {
	    tempvar uA_L5_`X' uB_L5_`X'
	}
	forvalues X = 1(1)$lambda6 {
	    tempvar uA_L6_`X' uB_L6_`X'
	}

	syntax varlist [if] [in] [fweight pweight] [, noLOg	Robust CLuster(varname)	svy	EForm *]
	marksample touse
	if "`svy'" != "" {
		svymarkout `touse'
	}
 
    quietly {

        * initialize horizon and payoffs
        generate double `ra' 	= $ML_y6
        generate double `t' 	= $ML_y2/360
        generate double `tau' 	= (30+$ML_y2)/360
        generate double `mA'	= $ML_y3
        generate double `mB'	= $ML_y4	
        generate double `prob1' = $ML_y5      
        generate double `prob2' = 1 - `prob1'
        generate double `mA1' 	= $ML_y7      
        generate double `mA2' 	= $ML_y8
        generate double `mB1' 	= $ML_y9
        generate double `mB2' 	= $ML_y10
	generate double `task1' = $ML_y11
	generate double `task2' = $ML_y12
	generate double `task3' = $ML_y13
	generate double `task4' = $ML_y14
	generate double `task5' = $ML_y15
	generate double `task6' = $ML_y16

        * transform parameters
        generate double `delta'  = exp(`LNdelta')
        generate double `alphaD' = exp(`LNalphaD')

        * estimating over risk responses
        generate double `yA1' = (($omega+`mA1')^(1-`r'))/(1-`r')	if `ra'==1
        generate double `yA2' = (($omega+`mA2')^(1-`r'))/(1-`r')	if `ra'==1
        generate double `yB1' = (($omega+`mB1')^(1-`r'))/(1-`r')	if `ra'==1
        generate double `yB2' = (($omega+`mB2')^(1-`r'))/(1-`r')	if `ra'==1

        generate double `wp1' = `prob1'					if `ra'==1	
        generate double `wp2' = `prob2'					if `ra'==1
        generate double `euA' = (`wp1'*`yA1')+(`wp2'*`yA2')		if `ra'==1      
        generate double `euB' = (`wp1'*`yB1')+(`wp2'*`yB2')		if `ra'==1

        generate double `euRatio' = (`euA'^(1/`muRA'))/((`euA'^(1/`muRA'))+(`euB'^(1/`muRA')))	if `ra'==1

        replace `lnf' = ln(`euRatio')					if $ML_y1==0	& `ra'==1
        replace `lnf' = ln(1-`euRatio')					if $ML_y1==1	& `ra'==1
        replace `lnf' = (ln(1-`euRatio')/2)+(ln(`euRatio')/2)		if $ML_y1==-1	& `ra'==1

        * estimating over time responses
        generate double `y0' = 0
        generate double `y1' = 0

	* Exponential discounting
	forvalues X = 1(1)$lambda0 {
	    generate double `uA_S_`X'' = (1 / ((1+`delta')^(30/360 + (`X'-1)/360 )) ) * (($omega + `mA'/$lambda0)^(1-`r'))/(1-`r')	if `ra'==0
	    generate double `uB_S_`X'' = (1 / ((1+`delta')^(30/360 + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0

	    replace `y0' = `y0' +  `uA_S_`X''		if `ra'==0
	    replace `y1' = `y1' +  `uB_S_`X''		if `ra'==0
	}	

	forvalues X = 1(1)$lambda1 {
	    generate double `uA_L1_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task1'==1
	    generate double `uB_L1_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda1)^(1-`r'))/(1-`r')	if `ra'==0 & `task1'==1

	    replace `y0' = `y0'  +  `uA_L1_`X''		if `ra'==0 & `task1'==1
	    replace `y1' = `y1'  +  `uB_L1_`X''		if `ra'==0 & `task1'==1
	}	

	forvalues X = 1(1)$lambda2 {
	    generate double `uA_L2_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task2'==1
	    generate double `uB_L2_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda2)^(1-`r'))/(1-`r')	if `ra'==0 & `task2'==1

	    replace `y0' = `y0'  +  `uA_L2_`X''		if `ra'==0 & `task2'==1
	    replace `y1' = `y1'  +  `uB_L2_`X''		if `ra'==0 & `task2'==1
	}	

	forvalues X = 1(1)$lambda3 {
	    generate double `uA_L3_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task3'==1
	    generate double `uB_L3_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda3)^(1-`r'))/(1-`r')	if `ra'==0 & `task3'==1

	    replace `y0' = `y0'  +  `uA_L3_`X''		if `ra'==0 & `task3'==1
	    replace `y1' = `y1'  +  `uB_L3_`X''		if `ra'==0 & `task3'==1
	}	

	forvalues X = 1(1)$lambda4 {
	    generate double `uA_L4_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task4'==1
	    generate double `uB_L4_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda4)^(1-`r'))/(1-`r')	if `ra'==0 & `task4'==1

	    replace `y0' = `y0'  +  `uA_L4_`X''		if `ra'==0 & `task4'==1
	    replace `y1' = `y1'  +  `uB_L4_`X''		if `ra'==0 & `task4'==1
	}	

	forvalues X = 1(1)$lambda5 {
	    generate double `uA_L5_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task5'==1
	    generate double `uB_L5_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda5)^(1-`r'))/(1-`r')	if `ra'==0 & `task5'==1

	    replace `y0' = `y0'  +  `uA_L5_`X''		if `ra'==0 & `task5'==1
	    replace `y1' = `y1'  +  `uB_L5_`X''		if `ra'==0 & `task5'==1
	}	

	forvalues X = 1(1)$lambda6 {
	    generate double `uA_L6_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task6'==1
	    generate double `uB_L6_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda6)^(1-`r'))/(1-`r')	if `ra'==0 & `task6'==1

	    replace `y0' = `y0'  +  `uA_L6_`X''		if `ra'==0 & `task6'==1
	    replace `y1' = `y1'  +  `uB_L6_`X''		if `ra'==0 & `task6'==1
	}	

        generate double `uRatio' = (`y0'^(1/`muIDR'))/((`y0'^(1/`muIDR'))+(`y1'^(1/`muIDR'))) 	if `ra'==0

        generate double `lnfEX' = ln(`uRatio')					if 	$ML_y1==0	& `ra'==0
        replace         `lnfEX' = ln(1-`uRatio')				if 	$ML_y1==1	& `ra'==0
        replace         `lnfEX' = (ln(1-`uRatio')/2)+(ln(`uRatio')/2)		if	$ML_y1==-1	& `ra'==0

	* Mazur-hyperbolic discounting
	forvalues X = 1(1)$lambda0 {
	    replace `uA_S_`X'' = (1 / (1+`alphaD'*(30/360 + (`X'-1)/360)) ) * (($omega + `mA'/$lambda0)^(1-`r'))/(1-`r')	if `ra'==0
	    replace `uB_S_`X'' = (1 / (1+`alphaD'*(30/360 + (`X'-1)/360)) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0

	    replace `y0' = `y0' +  `uA_S_`X''		if `ra'==0
	    replace `y1' = `y1' +  `uB_S_`X''		if `ra'==0
	}	

	forvalues X = 1(1)$lambda1 {
	    replace `uA_L1_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task1'==1
	    replace `uB_L1_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda1)^(1-`r'))/(1-`r')	if `ra'==0 & `task1'==1

	    replace `y0' = `y0'  +  `uA_L1_`X''		if `ra'==0 & `task1'==1
	    replace `y1' = `y1'  +  `uB_L1_`X''		if `ra'==0 & `task1'==1
	}	

	forvalues X = 1(1)$lambda2 {
	    replace `uA_L2_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task2'==1
	    replace `uB_L2_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda2)^(1-`r'))/(1-`r')	if `ra'==0 & `task2'==1

	    replace `y0' = `y0'  +  `uA_L2_`X''		if `ra'==0 & `task2'==1
	    replace `y1' = `y1'  +  `uB_L2_`X''		if `ra'==0 & `task2'==1
	}	

	forvalues X = 1(1)$lambda3 {
	    replace `uA_L3_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task3'==1
	    replace `uB_L3_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda3)^(1-`r'))/(1-`r')	if `ra'==0 & `task3'==1

	    replace `y0' = `y0'  +  `uA_L3_`X''		if `ra'==0 & `task3'==1
	    replace `y1' = `y1'  +  `uB_L3_`X''		if `ra'==0 & `task3'==1
	}	

	forvalues X = 1(1)$lambda4 {
	    replace `uA_L4_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task4'==1
	    replace `uB_L4_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda4)^(1-`r'))/(1-`r')	if `ra'==0 & `task4'==1

	    replace `y0' = `y0'  +  `uA_L4_`X''		if `ra'==0 & `task4'==1
	    replace `y1' = `y1'  +  `uB_L4_`X''		if `ra'==0 & `task4'==1
	}	

	forvalues X = 1(1)$lambda5 {
	    replace `uA_L5_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task5'==1
	    replace `uB_L5_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda5)^(1-`r'))/(1-`r')	if `ra'==0 & `task5'==1

	    replace `y0' = `y0'  +  `uA_L5_`X''		if `ra'==0 & `task5'==1
	    replace `y1' = `y1'  +  `uB_L5_`X''		if `ra'==0 & `task5'==1
	}	

	forvalues X = 1(1)$lambda6 {
	    replace `uA_L6_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task6'==1
	    replace `uB_L6_`X'' = (1 / (1+`alphaD'*(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda6)^(1-`r'))/(1-`r')	if `ra'==0 & `task6'==1

	    replace `y0' = `y0'  +  `uA_L6_`X''		if `ra'==0 & `task6'==1
	    replace `y1' = `y1'  +  `uB_L6_`X''		if `ra'==0 & `task6'==1
	}	

        replace `uRatio' = (`y0'^(1/`muIDR'))/((`y0'^(1/`muIDR'))+(`y1'^(1/`muIDR'))) 	if `ra'==0

        generate double `lnfPH' = ln(`uRatio')					if 	$ML_y1==0	& `ra'==0
        replace         `lnfPH' = ln(1-`uRatio')				if 	$ML_y1==1	& `ra'==0
        replace         `lnfPH' = (ln(1-`uRatio')/2)+(ln(`uRatio')/2)		if	$ML_y1==-1	& `ra'==0

        * get the grand index for the discounting choices
        generate double `f1' = exp(`lnfEX')					if `ra'==0
        generate double `f2' = exp(`lnfPH')					if `ra'==0
        generate double `p1' =            1/(1+exp(`kappa'))			if `ra'==0
        generate double `p2' = exp(`kappa')/(1+exp(`kappa'))			if `ra'==0
        replace	`lnf' = ln((`p1' * `f1') + (`p2' * `f2'))			if `ra'==0

    }

end

* Mixture model of two exponential discounting specifications with CRRA
program MLmix_ex_ex_lam

    args lnf r LNdelta LNalphaD kappa muRA muIDR

    tempvar ra t mA mB prob1 prob2 mA1 mA2 mB1 mB2 yA1 yA2 yB1 yB2 wp1 wp2 euA euB euRatio y0 y1 
    tempvar tau mA mB uA uB uRatio baseSR baseMR rMR scale delta alphaD lnfEX lnfPH f1 f2 p1 p2
    tempvar task1 task2 task3 task4 task5 task6

    * Define temporary variables
	forvalues X = 1(1)$lambda0 {
	    tempvar uA_S_`X' uB_S_`X'
	}
	forvalues X = 1(1)$lambda1 {
	    tempvar uA_L1_`X' uB_L1_`X'
	}
	forvalues X = 1(1)$lambda2 {
	    tempvar uA_L2_`X' uB_L2_`X'
	}
	forvalues X = 1(1)$lambda3 {
	    tempvar uA_L3_`X' uB_L3_`X'
	}
	forvalues X = 1(1)$lambda4 {
	    tempvar uA_L4_`X' uB_L4_`X'
	}
	forvalues X = 1(1)$lambda5 {
	    tempvar uA_L5_`X' uB_L5_`X'
	}
	forvalues X = 1(1)$lambda6 {
	    tempvar uA_L6_`X' uB_L6_`X'
	}

	syntax varlist [if] [in] [fweight pweight] [, noLOg	Robust CLuster(varname)	svy	EForm *]
	marksample touse
	if "`svy'" != "" {
		svymarkout `touse'
	}
 
    quietly {

        * initialize horizon and payoffs
        generate double `ra' 	= $ML_y6
        generate double `t' 	= $ML_y2/360
        generate double `tau' 	= (30+$ML_y2)/360
        generate double `mA'	= $ML_y3
        generate double `mB'	= $ML_y4	
        generate double `prob1' = $ML_y5      
        generate double `prob2' = 1 - `prob1'
        generate double `mA1' 	= $ML_y7      
        generate double `mA2' 	= $ML_y8
        generate double `mB1' 	= $ML_y9
        generate double `mB2' 	= $ML_y10
	generate double `task1' = $ML_y11
	generate double `task2' = $ML_y12
	generate double `task3' = $ML_y13
	generate double `task4' = $ML_y14
	generate double `task5' = $ML_y15
	generate double `task6' = $ML_y16

        * transform parameters
        generate double `delta'  = exp(`LNdelta')
        generate double `alphaD' = exp(`LNalphaD')

        * estimating over risk responses
        generate double `yA1' = (($omega+`mA1')^(1-`r'))/(1-`r')	if `ra'==1
        generate double `yA2' = (($omega+`mA2')^(1-`r'))/(1-`r')	if `ra'==1
        generate double `yB1' = (($omega+`mB1')^(1-`r'))/(1-`r')	if `ra'==1
        generate double `yB2' = (($omega+`mB2')^(1-`r'))/(1-`r')	if `ra'==1

        generate double `wp1' = `prob1'					if `ra'==1	
        generate double `wp2' = `prob2'					if `ra'==1
        generate double `euA' = (`wp1'*`yA1')+(`wp2'*`yA2')		if `ra'==1      
        generate double `euB' = (`wp1'*`yB1')+(`wp2'*`yB2')		if `ra'==1

        generate double `euRatio' = (`euA'^(1/`muRA'))/((`euA'^(1/`muRA'))+(`euB'^(1/`muRA')))	if `ra'==1

        replace `lnf' = ln(`euRatio')					if $ML_y1==0	& `ra'==1
        replace `lnf' = ln(1-`euRatio')					if $ML_y1==1	& `ra'==1
        replace `lnf' = (ln(1-`euRatio')/2)+(ln(`euRatio')/2)		if $ML_y1==-1	& `ra'==1

        * estimating over time responses
        generate double `y0' = 0
        generate double `y1' = 0

	* Exponential discounting
	forvalues X = 1(1)$lambda0 {
	    generate double `uA_S_`X'' = (1 / ((1+`delta')^(30/360 + (`X'-1)/360 )) ) * (($omega + `mA'/$lambda0)^(1-`r'))/(1-`r')	if `ra'==0
	    generate double `uB_S_`X'' = (1 / ((1+`delta')^(30/360 + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0

	    replace `y0' = `y0' +  `uA_S_`X''		if `ra'==0
	    replace `y1' = `y1' +  `uB_S_`X''		if `ra'==0
	}	

	forvalues X = 1(1)$lambda1 {
	    generate double `uA_L1_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task1'==1
	    generate double `uB_L1_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda1)^(1-`r'))/(1-`r')	if `ra'==0 & `task1'==1

	    replace `y0' = `y0'  +  `uA_L1_`X''		if `ra'==0 & `task1'==1
	    replace `y1' = `y1'  +  `uB_L1_`X''		if `ra'==0 & `task1'==1
	}	

	forvalues X = 1(1)$lambda2 {
	    generate double `uA_L2_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task2'==1
	    generate double `uB_L2_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda2)^(1-`r'))/(1-`r')	if `ra'==0 & `task2'==1

	    replace `y0' = `y0'  +  `uA_L2_`X''		if `ra'==0 & `task2'==1
	    replace `y1' = `y1'  +  `uB_L2_`X''		if `ra'==0 & `task2'==1
	}	

	forvalues X = 1(1)$lambda3 {
	    generate double `uA_L3_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task3'==1
	    generate double `uB_L3_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda3)^(1-`r'))/(1-`r')	if `ra'==0 & `task3'==1

	    replace `y0' = `y0'  +  `uA_L3_`X''		if `ra'==0 & `task3'==1
	    replace `y1' = `y1'  +  `uB_L3_`X''		if `ra'==0 & `task3'==1
	}	

	forvalues X = 1(1)$lambda4 {
	    generate double `uA_L4_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task4'==1
	    generate double `uB_L4_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda4)^(1-`r'))/(1-`r')	if `ra'==0 & `task4'==1

	    replace `y0' = `y0'  +  `uA_L4_`X''		if `ra'==0 & `task4'==1
	    replace `y1' = `y1'  +  `uB_L4_`X''		if `ra'==0 & `task4'==1
	}	

	forvalues X = 1(1)$lambda5 {
	    generate double `uA_L5_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task5'==1
	    generate double `uB_L5_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda5)^(1-`r'))/(1-`r')	if `ra'==0 & `task5'==1

	    replace `y0' = `y0'  +  `uA_L5_`X''		if `ra'==0 & `task5'==1
	    replace `y1' = `y1'  +  `uB_L5_`X''		if `ra'==0 & `task5'==1
	}	

	forvalues X = 1(1)$lambda6 {
	    generate double `uA_L6_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task6'==1
	    generate double `uB_L6_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda6)^(1-`r'))/(1-`r')	if `ra'==0 & `task6'==1

	    replace `y0' = `y0'  +  `uA_L6_`X''		if `ra'==0 & `task6'==1
	    replace `y1' = `y1'  +  `uB_L6_`X''		if `ra'==0 & `task6'==1
	}	

        generate double `uRatio' = (`y0'^(1/`muIDR'))/((`y0'^(1/`muIDR'))+(`y1'^(1/`muIDR'))) 	if `ra'==0

        generate double `lnfEX' = ln(`uRatio')					if 	$ML_y1==0	& `ra'==0
        replace         `lnfEX' = ln(1-`uRatio')				if 	$ML_y1==1	& `ra'==0
        replace         `lnfEX' = (ln(1-`uRatio')/2)+(ln(`uRatio')/2)		if	$ML_y1==-1	& `ra'==0

	* Exponential discounting
	forvalues X = 1(1)$lambda0 {
	    replace `uA_S_`X'' = (1 / ((1+`alphaD')^(30/360 + (`X'-1)/360 )) ) * (($omega + `mA'/$lambda0)^(1-`r'))/(1-`r')	if `ra'==0
	    replace `uB_S_`X'' = (1 / ((1+`alphaD')^(30/360 + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0

	    replace `y0' = `y0' +  `uA_S_`X''		if `ra'==0
	    replace `y1' = `y1' +  `uB_S_`X''		if `ra'==0
	}	

	forvalues X = 1(1)$lambda1 {
	    replace `uA_L1_`X'' = (1 / ((1+`alphaD')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task1'==1
	    replace `uB_L1_`X'' = (1 / ((1+`alphaD')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda1)^(1-`r'))/(1-`r')	if `ra'==0 & `task1'==1

	    replace `y0' = `y0'  +  `uA_L1_`X''		if `ra'==0 & `task1'==1
	    replace `y1' = `y1'  +  `uB_L1_`X''		if `ra'==0 & `task1'==1
	}	

	forvalues X = 1(1)$lambda2 {
	    replace `uA_L2_`X'' = (1 / ((1+`alphaD')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task2'==1
	    replace `uB_L2_`X'' = (1 / ((1+`alphaD')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda2)^(1-`r'))/(1-`r')	if `ra'==0 & `task2'==1

	    replace `y0' = `y0'  +  `uA_L2_`X''		if `ra'==0 & `task2'==1
	    replace `y1' = `y1'  +  `uB_L2_`X''		if `ra'==0 & `task2'==1
	}	

	forvalues X = 1(1)$lambda3 {
	    replace `uA_L3_`X'' = (1 / ((1+`alphaD')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task3'==1
	    replace `uB_L3_`X'' = (1 / ((1+`alphaD')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda3)^(1-`r'))/(1-`r')	if `ra'==0 & `task3'==1

	    replace `y0' = `y0'  +  `uA_L3_`X''		if `ra'==0 & `task3'==1
	    replace `y1' = `y1'  +  `uB_L3_`X''		if `ra'==0 & `task3'==1
	}	

	forvalues X = 1(1)$lambda4 {
	    replace `uA_L4_`X'' = (1 / ((1+`alphaD')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task4'==1
	    replace `uB_L4_`X'' = (1 / ((1+`alphaD')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda4)^(1-`r'))/(1-`r')	if `ra'==0 & `task4'==1

	    replace `y0' = `y0'  +  `uA_L4_`X''		if `ra'==0 & `task4'==1
	    replace `y1' = `y1'  +  `uB_L4_`X''		if `ra'==0 & `task4'==1
	}	

	forvalues X = 1(1)$lambda5 {
	    replace `uA_L5_`X'' = (1 / ((1+`alphaD')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task5'==1
	    replace `uB_L5_`X'' = (1 / ((1+`alphaD')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda5)^(1-`r'))/(1-`r')	if `ra'==0 & `task5'==1

	    replace `y0' = `y0'  +  `uA_L5_`X''		if `ra'==0 & `task5'==1
	    replace `y1' = `y1'  +  `uB_L5_`X''		if `ra'==0 & `task5'==1
	}	

	forvalues X = 1(1)$lambda6 {
	    replace `uA_L6_`X'' = (1 / ((1+`alphaD')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task6'==1
	    replace `uB_L6_`X'' = (1 / ((1+`alphaD')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda6)^(1-`r'))/(1-`r')	if `ra'==0 & `task6'==1

	    replace `y0' = `y0'  +  `uA_L6_`X''		if `ra'==0 & `task6'==1
	    replace `y1' = `y1'  +  `uB_L6_`X''		if `ra'==0 & `task6'==1
	}	

        replace `uRatio' = (`y0'^(1/`muIDR'))/((`y0'^(1/`muIDR'))+(`y1'^(1/`muIDR'))) 	if `ra'==0

        generate double `lnfPH' = ln(`uRatio')					if 	$ML_y1==0	& `ra'==0
        replace         `lnfPH' = ln(1-`uRatio')				if 	$ML_y1==1	& `ra'==0
        replace         `lnfPH' = (ln(1-`uRatio')/2)+(ln(`uRatio')/2)		if	$ML_y1==-1	& `ra'==0

        * get the grand index for the discounting choices
        generate double `f1' = exp(`lnfEX')					if `ra'==0
        generate double `f2' = exp(`lnfPH')					if `ra'==0
        generate double `p1' =            1/(1+exp(`kappa'))			if `ra'==0
        generate double `p2' = exp(`kappa')/(1+exp(`kappa'))			if `ra'==0
        replace	`lnf' = ln((`p1' * `f1') + (`p2' * `f2'))			if `ra'==0

    }

end

* Mixture model of two exponential discounting specifications with CRRA
program MLmix_ex_ex_lam2

    args lnf r LNdelta LNalphaD kappa muRA muIDR

    tempvar ra t mA mB prob1 prob2 mA1 mA2 mB1 mB2 yA1 yA2 yB1 yB2 wp1 wp2 euA euB euRatio y0 y1 
    tempvar tau mA mB uA uB uRatio baseSR baseMR rMR scale delta alphaD lnfEX lnfPH f1 f2 p1 p2
    tempvar task1 task2 task3 task4 task5 task6

    * Define temporary variables
	forvalues X = 1(1)$lambda0a {
	    tempvar uA_Sa_`X' uB_Sa_`X'
	}
	forvalues X = 1(1)$lambda1a {
	    tempvar uA_L1a_`X' uB_L1a_`X'
	}
	forvalues X = 1(1)$lambda2a {
	    tempvar uA_L2a_`X' uB_L2a_`X'
	}
	forvalues X = 1(1)$lambda3a {
	    tempvar uA_L3a_`X' uB_L3a_`X'
	}
	forvalues X = 1(1)$lambda4a {
	    tempvar uA_L4a_`X' uB_L4a_`X'
	}
	forvalues X = 1(1)$lambda5a {
	    tempvar uA_L5a_`X' uB_L5a_`X'
	}
	forvalues X = 1(1)$lambda6a {
	    tempvar uA_L6a_`X' uB_L6a_`X'
	}

	forvalues X = 1(1)$lambda0b {
	    tempvar uA_Sb_`X' uB_Sb_`X'
	}
	forvalues X = 1(1)$lambda1b {
	    tempvar uA_L1b_`X' uB_L1b_`X'
	}
	forvalues X = 1(1)$lambda2b {
	    tempvar uA_L2b_`X' uB_L2b_`X'
	}
	forvalues X = 1(1)$lambda3b {
	    tempvar uA_L3b_`X' uB_L3b_`X'
	}
	forvalues X = 1(1)$lambda4b {
	    tempvar uA_L4b_`X' uB_L4b_`X'
	}
	forvalues X = 1(1)$lambda5b {
	    tempvar uA_L5b_`X' uB_L5b_`X'
	}
	forvalues X = 1(1)$lambda6b {
	    tempvar uA_L6b_`X' uB_L6b_`X'
	}

	syntax varlist [if] [in] [fweight pweight] [, noLOg	Robust CLuster(varname)	svy	EForm *]
	marksample touse
	if "`svy'" != "" {
		svymarkout `touse'
	}
 
    quietly {

        * initialize horizon and payoffs
        generate double `ra' 	= $ML_y6
        generate double `t' 	= $ML_y2/360
        generate double `tau' 	= (30+$ML_y2)/360
        generate double `mA'	= $ML_y3
        generate double `mB'	= $ML_y4	
        generate double `prob1' = $ML_y5      
        generate double `prob2' = 1 - `prob1'
        generate double `mA1' 	= $ML_y7      
        generate double `mA2' 	= $ML_y8
        generate double `mB1' 	= $ML_y9
        generate double `mB2' 	= $ML_y10
	generate double `task1' = $ML_y11
	generate double `task2' = $ML_y12
	generate double `task3' = $ML_y13
	generate double `task4' = $ML_y14
	generate double `task5' = $ML_y15
	generate double `task6' = $ML_y16

        * transform parameters
        generate double `delta'  = exp(`LNdelta')
        generate double `alphaD' = exp(`LNalphaD')

        * estimating over risk responses
        generate double `yA1' = (($omega+`mA1')^(1-`r'))/(1-`r')	if `ra'==1
        generate double `yA2' = (($omega+`mA2')^(1-`r'))/(1-`r')	if `ra'==1
        generate double `yB1' = (($omega+`mB1')^(1-`r'))/(1-`r')	if `ra'==1
        generate double `yB2' = (($omega+`mB2')^(1-`r'))/(1-`r')	if `ra'==1

        generate double `wp1' = `prob1'					if `ra'==1	
        generate double `wp2' = `prob2'					if `ra'==1
        generate double `euA' = (`wp1'*`yA1')+(`wp2'*`yA2')		if `ra'==1      
        generate double `euB' = (`wp1'*`yB1')+(`wp2'*`yB2')		if `ra'==1

        generate double `euRatio' = (`euA'^(1/`muRA'))/((`euA'^(1/`muRA'))+(`euB'^(1/`muRA')))	if `ra'==1

        replace `lnf' = ln(`euRatio')					if $ML_y1==0	& `ra'==1
        replace `lnf' = ln(1-`euRatio')					if $ML_y1==1	& `ra'==1
        replace `lnf' = (ln(1-`euRatio')/2)+(ln(`euRatio')/2)		if $ML_y1==-1	& `ra'==1

        * estimating over time responses
        generate double `y0' = 0
        generate double `y1' = 0

	* Exponential discounting
	forvalues X = 1(1)$lambda0a {
	    generate double `uA_Sa_`X'' = (1 / ((1+`delta')^(30/360 + (`X'-1)/360 )) ) * (($omega + `mA'/$lambda0a)^(1-`r'))/(1-`r')	if `ra'==0
	    generate double `uB_Sa_`X'' = (1 / ((1+`delta')^(30/360 + (`X'-1)/360 )) ) * (($omega                 )^(1-`r'))/(1-`r')	if `ra'==0

	    replace `y0' = `y0' +  `uA_Sa_`X''		if `ra'==0
	    replace `y1' = `y1' +  `uB_Sa_`X''		if `ra'==0
	}	

	forvalues X = 1(1)$lambda1a {
	    generate double `uA_L1a_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                 )^(1-`r'))/(1-`r')	if `ra'==0 & `task1'==1
	    generate double `uB_L1a_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda1a)^(1-`r'))/(1-`r')	if `ra'==0 & `task1'==1

	    replace `y0' = `y0'  +  `uA_L1a_`X''		if `ra'==0 & `task1'==1
	    replace `y1' = `y1'  +  `uB_L1a_`X''		if `ra'==0 & `task1'==1
	}	

	forvalues X = 1(1)$lambda2a {
	    generate double `uA_L2a_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                 )^(1-`r'))/(1-`r')	if `ra'==0 & `task2'==1
	    generate double `uB_L2a_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda2a)^(1-`r'))/(1-`r')	if `ra'==0 & `task2'==1

	    replace `y0' = `y0'  +  `uA_L2a_`X''		if `ra'==0 & `task2'==1
	    replace `y1' = `y1'  +  `uB_L2a_`X''		if `ra'==0 & `task2'==1
	}	

	forvalues X = 1(1)$lambda3a {
	    generate double `uA_L3a_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                 )^(1-`r'))/(1-`r')	if `ra'==0 & `task3'==1
	    generate double `uB_L3a_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda3a)^(1-`r'))/(1-`r')	if `ra'==0 & `task3'==1

	    replace `y0' = `y0'  +  `uA_L3a_`X''		if `ra'==0 & `task3'==1
	    replace `y1' = `y1'  +  `uB_L3a_`X''		if `ra'==0 & `task3'==1
	}	

	forvalues X = 1(1)$lambda4a {
	    generate double `uA_L4a_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                 )^(1-`r'))/(1-`r')	if `ra'==0 & `task4'==1
	    generate double `uB_L4a_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda4a)^(1-`r'))/(1-`r')	if `ra'==0 & `task4'==1

	    replace `y0' = `y0'  +  `uA_L4a_`X''		if `ra'==0 & `task4'==1
	    replace `y1' = `y1'  +  `uB_L4a_`X''		if `ra'==0 & `task4'==1
	}	

	forvalues X = 1(1)$lambda5a {
	    generate double `uA_L5a_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                 )^(1-`r'))/(1-`r')	if `ra'==0 & `task5'==1
	    generate double `uB_L5a_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda5a)^(1-`r'))/(1-`r')	if `ra'==0 & `task5'==1

	    replace `y0' = `y0'  +  `uA_L5a_`X''		if `ra'==0 & `task5'==1
	    replace `y1' = `y1'  +  `uB_L5a_`X''		if `ra'==0 & `task5'==1
	}	

	forvalues X = 1(1)$lambda6a {
	    generate double `uA_L6a_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega                 )^(1-`r'))/(1-`r')	if `ra'==0 & `task6'==1
	    generate double `uB_L6a_`X'' = (1 / ((1+`delta')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda6a)^(1-`r'))/(1-`r')	if `ra'==0 & `task6'==1

	    replace `y0' = `y0'  +  `uA_L6a_`X''		if `ra'==0 & `task6'==1
	    replace `y1' = `y1'  +  `uB_L6a_`X''		if `ra'==0 & `task6'==1
	}	

        generate double `uRatio' = (`y0'^(1/`muIDR'))/((`y0'^(1/`muIDR'))+(`y1'^(1/`muIDR'))) 	if `ra'==0

        generate double `lnfEX' = ln(`uRatio')					if 	$ML_y1==0	& `ra'==0
        replace         `lnfEX' = ln(1-`uRatio')				if 	$ML_y1==1	& `ra'==0
        replace         `lnfEX' = (ln(1-`uRatio')/2)+(ln(`uRatio')/2)		if	$ML_y1==-1	& `ra'==0

	* Exponential discounting
	forvalues X = 1(1)$lambda0b {
	    generate double `uA_Sb_`X'' = (1 / ((1+`alphaD')^(30/360 + (`X'-1)/360 )) ) * (($omega + `mA'/$lambda0b)^(1-`r'))/(1-`r')	if `ra'==0
	    generate double `uB_Sb_`X'' = (1 / ((1+`alphaD')^(30/360 + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0

	    replace `y0' = `y0' +  `uA_Sb_`X''		if `ra'==0
	    replace `y1' = `y1' +  `uB_Sb_`X''		if `ra'==0
	}	

	forvalues X = 1(1)$lambda1b {
	    generate double `uA_L1b_`X'' = (1 / ((1+`alphaD')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task1'==1
	    generate double `uB_L1b_`X'' = (1 / ((1+`alphaD')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda1b)^(1-`r'))/(1-`r')	if `ra'==0 & `task1'==1

	    replace `y0' = `y0'  +  `uA_L1b_`X''		if `ra'==0 & `task1'==1
	    replace `y1' = `y1'  +  `uB_L1b_`X''		if `ra'==0 & `task1'==1
	}	

	forvalues X = 1(1)$lambda2b {
	    generate double `uA_L2b_`X'' = (1 / ((1+`alphaD')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task2'==1
	    generate double `uB_L2b_`X'' = (1 / ((1+`alphaD')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda2b)^(1-`r'))/(1-`r')	if `ra'==0 & `task2'==1

	    replace `y0' = `y0'  +  `uA_L2b_`X''		if `ra'==0 & `task2'==1
	    replace `y1' = `y1'  +  `uB_L2b_`X''		if `ra'==0 & `task2'==1
	}	

	forvalues X = 1(1)$lambda3b {
	    generate double `uA_L3b_`X'' = (1 / ((1+`alphaD')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task3'==1
	    generate double `uB_L3b_`X'' = (1 / ((1+`alphaD')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda3b)^(1-`r'))/(1-`r')	if `ra'==0 & `task3'==1

	    replace `y0' = `y0'  +  `uA_L3b_`X''		if `ra'==0 & `task3'==1
	    replace `y1' = `y1'  +  `uB_L3b_`X''		if `ra'==0 & `task3'==1
	}	

	forvalues X = 1(1)$lambda4b {
	    generate double `uA_L4b_`X'' = (1 / ((1+`alphaD')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task4'==1
	    generate double `uB_L4b_`X'' = (1 / ((1+`alphaD')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda4b)^(1-`r'))/(1-`r')	if `ra'==0 & `task4'==1

	    replace `y0' = `y0'  +  `uA_L4b_`X''		if `ra'==0 & `task4'==1
	    replace `y1' = `y1'  +  `uB_L4b_`X''		if `ra'==0 & `task4'==1
	}	

	forvalues X = 1(1)$lambda5b {
	    generate double `uA_L5b_`X'' = (1 / ((1+`alphaD')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task5'==1
	    generate double `uB_L5b_`X'' = (1 / ((1+`alphaD')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda5b)^(1-`r'))/(1-`r')	if `ra'==0 & `task5'==1

	    replace `y0' = `y0'  +  `uA_L5b_`X''		if `ra'==0 & `task5'==1
	    replace `y1' = `y1'  +  `uB_L5b_`X''		if `ra'==0 & `task5'==1
	}	

	forvalues X = 1(1)$lambda6b {
	    generate double `uA_L6b_`X'' = (1 / ((1+`alphaD')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r'))/(1-`r')	if `ra'==0 & `task6'==1
	    generate double `uB_L6b_`X'' = (1 / ((1+`alphaD')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda6b)^(1-`r'))/(1-`r')	if `ra'==0 & `task6'==1

	    replace `y0' = `y0'  +  `uA_L6b_`X''		if `ra'==0 & `task6'==1
	    replace `y1' = `y1'  +  `uB_L6b_`X''		if `ra'==0 & `task6'==1
	}	

        replace `uRatio' = (`y0'^(1/`muIDR'))/((`y0'^(1/`muIDR'))+(`y1'^(1/`muIDR'))) 	if `ra'==0

        generate double `lnfPH' = ln(`uRatio')					if 	$ML_y1==0	& `ra'==0
        replace         `lnfPH' = ln(1-`uRatio')				if 	$ML_y1==1	& `ra'==0
        replace         `lnfPH' = (ln(1-`uRatio')/2)+(ln(`uRatio')/2)		if	$ML_y1==-1	& `ra'==0

        * get the grand index for the discounting choices
        generate double `f1' = exp(`lnfEX')					if `ra'==0
        generate double `f2' = exp(`lnfPH')					if `ra'==0
        generate double `p1' =            1/(1+exp(`kappa'))			if `ra'==0
        generate double `p2' = exp(`kappa')/(1+exp(`kappa'))			if `ra'==0
        replace	`lnf' = ln((`p1' * `f1') + (`p2' * `f2'))			if `ra'==0

    }

end


** PROGRAMS TO PLOT FUNCTIONS **
program define GetEx
   qui {

	capture {
	forvalues Y=$HorizonLO($HorizonInc)$HorizonHI {
       		foreach V in ExDis ExDisLO ExDisHI {
               		generate s`V'`Y'=.
       		}
       	predictnl ExDis`Y'=exp(xb(LNdelta)), ci(ExDisLO`Y' ExDisHI`Y') iterate(1000)
       	}
	}

    	forvalues Y=$HorizonLO($HorizonInc)$HorizonHI {
        	foreach V in ExDis ExDisLO ExDisHI {
            		replace s`V'`Y'=`V'`Y' if _rc~=498
            		drop `V'`Y'
            		rename s`V'`Y' `V'`Y'
        	}
    	}

    * generate data for pooled pictures
    egen records=seq()
    keep if records==1
    reshape long ExDis ExDisLO ExDisHI, i(id) j(Horizon)
    replace Horizon = Horizon/$HorizonSCALE
    keep Horizon ExDis ExDisLO ExDisHI
    save ExDis, replace
}

end


program define GetEx2
   qui {
	capture {
	forvalues Y=$HorizonLO($HorizonInc)$HorizonHI {
       		foreach V in ExDis ExDisLO ExDisHI {
               		generate s`V'`Y'=.
       		}
       	predictnl ExDis`Y'=xb(delta), ci(ExDisLO`Y' ExDisHI`Y') iterate(1000)
       	}
	}

    	forvalues Y=$HorizonLO($HorizonInc)$HorizonHI {
        	foreach V in ExDis ExDisLO ExDisHI {
            		replace s`V'`Y'=`V'`Y' if _rc~=498
            		drop `V'`Y'
            		rename s`V'`Y' `V'`Y'
        	}
    	}

    * generate data for pooled pictures
    egen records=seq()
    keep if records==1
    reshape long ExDis ExDisLO ExDisHI, i(id) j(Horizon)
    replace Horizon = Horizon/$HorizonSCALE
    keep Horizon ExDis ExDisLO ExDisHI
    save ExDis, replace
}

end


program define GetHY
   qui {
	capture {
	forvalues Y=$HorizonLO($HorizonInc)$HorizonHI {
       		foreach V in HYDis HYDisLO HYDisHI {
               		generate s`V'`Y'=.
       		}
       		predictnl HYDis`Y'= (1+exp(xb(LNalphaD))*`Y')^(1/`Y') - 1, ci(HYDisLO`Y' HYDisHI`Y') iterate(1000)
       	}
	}

    	forvalues Y=$HorizonLO($HorizonInc)$HorizonHI {
        	foreach V in HYDis HYDisLO HYDisHI {
            		replace s`V'`Y'=`V'`Y' if _rc~=498
            		drop `V'`Y'
            		rename s`V'`Y' `V'`Y'
        	}
    	}

    * generate data for pooled pictures
    egen records=seq()
    keep if records==1
    reshape long HYDis HYDisLO HYDisHI, i(id) j(Horizon)
    replace Horizon = Horizon/$HorizonSCALE
    keep Horizon HYDis HYDisLO HYDisHI
    save HYDis, replace
}

end


* get discount rates for hyperbolic DI specification of Prelec [2004]
program define GetHY_
   qui {
	capture {
	forvalues Y=$HorizonLO($HorizonInc)$HorizonHI {
       		foreach V in HY_Dis HY_DisLO HY_DisHI {
               		generate s`V'`Y'=.
       		}
		predictnl HY_Dis`Y'= ($alphaDLO + ($alphaDHI-$alphaDLO)*(1/(1+exp(xb(alphaD_)))))*xb(betaD)*`Y'^(($alphaDLO + ($alphaDHI-$alphaDLO)*(1/(1+exp(xb(alphaD_)))))-1), ci(HY_DisLO`Y' HY_DisHI`Y') iterate(1000)
       	}
	}

    	forvalues Y=$HorizonLO($HorizonInc)$HorizonHI {
        	foreach V in HY_Dis HY_DisLO HY_DisHI {
            		replace s`V'`Y'=`V'`Y' if _rc~=498
            		drop `V'`Y'
            		rename s`V'`Y' `V'`Y'
        	}
    	}

    * generate data for pooled pictures
    egen records=seq()
    keep if records==1
    reshape long HY_Dis HY_DisLO HY_DisHI, i(id) j(Horizon)
    replace Horizon = Horizon/$HorizonSCALE
    keep Horizon HY_Dis HY_DisLO HY_DisHI
    save HY_Dis, replace
}

end

exit
