/* MLfunctions.do reads in all the handwritten ml functions */

** RANDOM EFFECTS MODELS **
* Overview of models *
*	MLex_rn_lam_r		Exponential model model with risk neutrality with ranom effects
*	MLex_ra_crra_lam_r	Exponential model with CRRA power specification and random effects


* Define exponential model with RN, FL dual self specification 
program MLex_rn_lam_r

    args lnf LNdelta muIDR LNdelta_sd

    * Define temporary variables
	forvalues X = 1(1)$lambda0 {
	    tempvar uA_S_`X'
	}
	forvalues X = 1(1)$lambda1 {
	    tempvar uB_L1_`X'
	}
	forvalues X = 1(1)$lambda2 {
	    tempvar uB_L2_`X'
	}
	forvalues X = 1(1)$lambda3 {
	    tempvar uB_L3_`X'
	}
	forvalues X = 1(1)$lambda4 {
	    tempvar uB_L4_`X'
	}
	forvalues X = 1(1)$lambda5 {
	    tempvar uB_L5_`X'
	}
	forvalues X = 1(1)$lambda6 {
	    tempvar uB_L6_`X'
	}

    tempvar t tau mA mB y0 y1 uRatio delta delta_sd 
    tempvar simmed_lnf sim_lnf delta_sim task1 task2 task3 task4 task5 task6

	syntax varlist [if] [in] [fweight pweight] [, noLOg	Robust CLuster(varname)	svy	EForm *]
	marksample touse
	if "`svy'" != "" {
		svymarkout `touse'
	}
 
    quietly {

        * initialize horizon and payoffs
        generate double `t' 	= $ML_y2/360
        generate double `tau' 	= (30+$ML_y2)/360
        generate double `mA'	= $ML_y3
        generate double `mB'	= $ML_y4	
        generate double `task1' = $ML_y11
        generate double `task2' = $ML_y12
        generate double `task3' = $ML_y13
        generate double `task4' = $ML_y14
        generate double `task5' = $ML_y15
        generate double `task6' = $ML_y16

        * transform parameters
        generate double `delta' = exp(`LNdelta')

  	
        * Generate parameters needed for the random effects
        scalar `delta_sd'	= exp(`LNdelta_sd')

       	* loop over likelihood evaluator with $draw random draws and accumulate simulated likelihoods
        generate double `simmed_lnf' = 0
        local draws = 0

        forvalues d=1/$draw {

            * generate bivariate normal or lognormal deviates
            if "$lognormal" == "n" {
                generate double `delta_sim'		= `delta'	+ `delta_sd'*    (invnormal(h1_`d'))

            }
            else {
                generate double `delta_sim'		= exp(`delta'	+ `delta_sd'* invnormal(h1_`d'))
            }

            * truncate if requested
            if "$truncate" == "y" {
                summ `delta_sim'
                local deltaM = r(mean)
                if `deltaM'>0 {
                    local draws = `draws'+1
                }
            }
            else {
                local deltaM = 1
                local draws = $draw
            }

            * estimating over time responses
            generate double `y0' = 0
            generate double `y1' = 0

            * estimating over time responses
            forvalues X = 1(1)$lambda0 {
                generate double `uA_S_`X'' = (1 / ((1+`delta_sim')^(30/360 + (`X'-1)/360 )) ) * (`mA'/$lambda0)
                replace `y0' = `y0' + `uA_S_`X''
                }	

            forvalues X = 1(1)$lambda1 {
                generate double `uB_L1_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (`mB'/$lambda1)		if `task1'==1
                replace `y1' = `y1'  +  `uB_L1_`X''		if `task1'==1
                }	

            forvalues X = 1(1)$lambda2 {
                generate double `uB_L2_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (`mB'/$lambda2)		if `task2'==1
                replace `y1' = `y1'  +  `uB_L2_`X''		if `task2'==1
                }	

            forvalues X = 1(1)$lambda3 {
                generate double `uB_L3_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (`mB'/$lambda3)		if `task3'==1
                replace `y1' = `y1'  +  `uB_L3_`X''		if `task3'==1
                }	

            forvalues X = 1(1)$lambda4 {
                generate double `uB_L4_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (`mB'/$lambda4)		if `task4'==1
                replace `y1' = `y1'  +  `uB_L4_`X''		if `task4'==1
                }	

            forvalues X = 1(1)$lambda5 {
                generate double `uB_L5_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (`mB'/$lambda5)		if `task5'==1
                replace `y1' = `y1'  +  `uB_L5_`X''		if `task5'==1
                }	

            forvalues X = 1(1)$lambda6 {
                generate double `uB_L6_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (`mB'/$lambda6)		if `task6'==1
                replace `y1' = `y1'  +  `uB_L6_`X''		if `task6'==1
                }	

            generate double `uRatio' = (`y0'^(1/`muIDR'))/((`y0'^(1/`muIDR'))+(`y1'^(1/`muIDR')))
            generate double `sim_lnf' = 0
            replace `sim_lnf' = ln(`uRatio')  				if  $ML_y1==0
            replace `sim_lnf' = ln(1-`uRatio')  			if  $ML_y1==1
            replace `sim_lnf' = (ln(1-`uRatio')/2)+(ln(`uRatio')/2)  	if  $ML_y1==-1

            * accumulate the simulated likelihood and probability (`delta'>0 operative only if $truncated in effect)
            if `deltaM'>0 {
                replace `simmed_lnf' 	= `simmed_lnf' + exp(`sim_lnf')
            }

            drop `sim_lnf' `delta_sim' `uRatio' `y0' `y1'
            forvalues X = 1(1)$lambda0 {
                drop `uA_S_`X''
            }	

            forvalues X = 1(1)$lambda1 {
                drop `uB_L1_`X''
            }	

            forvalues X = 1(1)$lambda2 {
                drop `uB_L2_`X''
            }	

            forvalues X = 1(1)$lambda3 {
                drop `uB_L3_`X''
            }	

            forvalues X = 1(1)$lambda4 {
                drop `uB_L4_`X''
            }	

            forvalues X = 1(1)$lambda5 {
                drop `uB_L5_`X''
            }	

            forvalues X = 1(1)$lambda6 {
                drop `uB_L6_`X''
            }
 
      }

      * get the average of the simulated draws
      replace `lnf' =  ln(`simmed_lnf'/`draws')

    }

end

* Define exponential model with CRRA, FL dual self specification (use delta here instead of LNdelta)
program MLex_ra_crra_lam_r

    args lnf r LNdelta muRA muIDR LNr_sd LNdelta_sd atrho12

	forvalues X = 1(1)$lambda0 {
	    tempvar uA_S_`X' uB_S_`X'
	}
	forvalues X = 1(1)$lambda1 {
	    tempvar uA_L1_`X' uB_L1_`X'
	}
	forvalues X = 1(1)$lambda2 {
	    tempvar uA_L2_`X' uB_L2_`X'
	}
	forvalues X = 1(1)$lambda3 {
	    tempvar uA_L3_`X' uB_L3_`X'
	}
	forvalues X = 1(1)$lambda4 {
	    tempvar uA_L4_`X' uB_L4_`X'
	}
	forvalues X = 1(1)$lambda5 {
	    tempvar uA_L5_`X' uB_L5_`X'
	}
	forvalues X = 1(1)$lambda6 {
	    tempvar uA_L6_`X' uB_L6_`X'
	}

    tempvar t tau mA mB y0 y1 uRatio delta delta_sd

    tempvar ra r_sd prob1 prob2 mA1 mA2 mB1 mB2 euA euB euRatio cov12 yA1 yA2 yB1 yB2 wp1 wp2

    tempvar simmed_lnf sim_lnf delta_sim r_sim task1 task2 task3 task4 task5 task6 sigma_r sigma_d

	syntax varlist [if] [in] [fweight pweight] [, noLOg	Robust CLuster(varname)	svy	EForm *]
	marksample touse
	if "`svy'" != "" {
		svymarkout `touse'
	}
 
    quietly {

        * initialize horizon and payoffs
        generate double `t' 	= $ML_y2/360
        generate double `tau' 	= (30+$ML_y2)/360
        generate double `mA'	= $ML_y3
        generate double `mB'	= $ML_y4	
        generate double `prob1' = $ML_y5      
        generate double `prob2' = 1 - `prob1'
        generate double `ra' 	= $ML_y6
        generate double `mA1' 	= $ML_y7      
        generate double `mA2' 	= $ML_y8
        generate double `mB1' 	= $ML_y9
        generate double `mB2' 	= $ML_y10
        generate double `task1' = $ML_y11
        generate double `task2' = $ML_y12
        generate double `task3' = $ML_y13
        generate double `task4' = $ML_y14
        generate double `task5' = $ML_y15
        generate double `task6' = $ML_y16

        * transform parameters
        generate double `delta'     = exp(`LNdelta')
        scalar `r_sd'               = exp(`LNr_sd')
        scalar `delta_sd'           = exp(`LNdelta_sd')

        * Generate parameters needed for the random effects
        scalar `sigma_r'	=(exp(`LNr_sd'))^2
        scalar `sigma_d'	=(exp(`LNdelta_sd'))^2
        scalar `cov12'		=[exp(2*`atrho12')-1]/[exp(2*`atrho12')+1]* (exp(`LNr_sd'))*(exp(`LNdelta_sd'))

        * define covariance matrix
    	matrix p = (`sigma_r', `cov12' \ `cov12', `sigma_d')
        $noise matrix list p
	
        * get the Cholesky factors calculated
        capture matrix c = cholesky(p)
       	if _rc ~= 0 {
            $noise display "Warning -- matrix probably not positive definite: using last matrix" 
        }
        else {
            $noise matrix list p
            $noise matrix list c
        }
        local c11 = c[1,1]
        local c21 = c[2,1]
        local c22 = c[2,2]
    	$noise display "Cholesky factors: c11 is `c11' c21 is `c21' and c22 is `c22'"

       	* loop over likelihood evaluator with $draw random draws and accumulate simulated likelihoods
        generate double `simmed_lnf' = 0
        local draws = 0

        forvalues d=1/$draw {
	
                * generate bivariate normal or lognormal deviates
                if "$lognormal" == "n" {
                    generate double `r_sim'		= `r'		+ `c11'*invnormal(h1_`d')
                    generate double `delta_sim'		= `delta'	+ `c21'*invnormal(h1_`d') + `c22'*invnormal(h2_`d')
                }
                else {
                    generate double `r_sim'		= exp(`r'		                  + `c11'invnormal(h1_`d'))
                    generate double `delta_sim'		= exp(`delta'	+ `c21'*invnormal(h1_`d') + `c22'*invnormal(h2_`d'))
                }

                * truncate if requested
                if "$truncate" == "y" {
                    summ `delta_sim'
                    local deltaM = r(mean)
                    if `deltaM'>0 {
                        local draws = `draws'+1
                    }
                }
                else {
                    local deltaM = 1
                    local draws = $draw
                }

                * estimating over risk responses
                    generate double `yA1' = (($omega+`mA1')^(1-`r_sim'))/(1-`r_sim')		if `ra'==1
                    generate double `yA2' = (($omega+`mA2')^(1-`r_sim'))/(1-`r_sim')		if `ra'==1
                    generate double `yB1' = (($omega+`mB1')^(1-`r_sim'))/(1-`r_sim')		if `ra'==1
                    generate double `yB2' = (($omega+`mB2')^(1-`r_sim'))/(1-`r_sim')		if `ra'==1

                generate double `wp1' = `prob1'					if `ra'==1	
                generate double `wp2' = `prob2'					if `ra'==1
                generate double `euA' = (`wp1'*`yA1')+(`wp2'*`yA2')		if `ra'==1      
                generate double `euB' = (`wp1'*`yB1')+(`wp2'*`yB2')		if `ra'==1

                generate double `euRatio' = (`euA'^(1/`muRA'))/((`euA'^(1/`muRA'))+(`euB'^(1/`muRA')))	if `ra'==1
 
            	generate double `sim_lnf' = 0
            	replace `sim_lnf' = ln(`euRatio')   				if $ML_y1==0		& `ra'==1
            	replace `sim_lnf' = ln(1-`euRatio')                                	if $ML_y1==1		& `ra'==1
            	replace `sim_lnf' = (ln(1-`euRatio')/2)+(ln(`euRatio')/2)		if $ML_y1==-1		& `ra'==1

                * estimating over time responses
                generate double `y0' = 0
                generate double `y1' = 0

                * estimating over time responses
                forvalues X = 1(1)$lambda0 {
                    generate double `uA_S_`X'' = (1 / ((1+`delta_sim')^(30/360 + (`X'-1)/360 )) ) * (($omega + `mA'/$lambda0)^(1-`r_sim'))/(1-`r_sim')	if `ra'==0
                    generate double `uB_S_`X'' = (1 / ((1+`delta_sim')^(30/360 + (`X'-1)/360 )) ) * (($omega                )^(1-`r_sim'))/(1-`r_sim')	if `ra'==0
                    replace `y0' = `y0' +  `uA_S_`X''		if `ra'==0
                    replace `y1' = `y1' +  `uB_S_`X''		if `ra'==0
                }	

                forvalues X = 1(1)$lambda1 {
                    generate double `uA_L1_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r_sim'))/(1-`r_sim')	if `ra'==0 & `task1'==1
                    generate double `uB_L1_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda1)^(1-`r_sim'))/(1-`r_sim')	if `ra'==0 & `task1'==1
                    replace `y0' = `y0'  +  `uA_L1_`X''		if `ra'==0 & `task1'==1
                    replace `y1' = `y1'  +  `uB_L1_`X''		if `ra'==0 & `task1'==1
                }	

                forvalues X = 1(1)$lambda2 {
                    generate double `uA_L2_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r_sim'))/(1-`r_sim')	if `ra'==0 & `task2'==1
                    generate double `uB_L2_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda2)^(1-`r_sim'))/(1-`r_sim')	if `ra'==0 & `task2'==1
                    replace `y0' = `y0'  +  `uA_L2_`X''		if `ra'==0 & `task2'==1
                    replace `y1' = `y1'  +  `uB_L2_`X''		if `ra'==0 & `task2'==1
                }	

                forvalues X = 1(1)$lambda3 {
                    generate double `uA_L3_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r_sim'))/(1-`r_sim')	if `ra'==0 & `task3'==1
                    generate double `uB_L3_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda3)^(1-`r_sim'))/(1-`r_sim')	if `ra'==0 & `task3'==1
                    replace `y0' = `y0'  +  `uA_L3_`X''		if `ra'==0 & `task3'==1
                    replace `y1' = `y1'  +  `uB_L3_`X''		if `ra'==0 & `task3'==1
                }	

                forvalues X = 1(1)$lambda4 {
                    generate double `uA_L4_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r_sim'))/(1-`r_sim')	if `ra'==0 & `task4'==1
                    generate double `uB_L4_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda4)^(1-`r_sim'))/(1-`r_sim')	if `ra'==0 & `task4'==1
                    replace `y0' = `y0'  +  `uA_L4_`X''		if `ra'==0 & `task4'==1
                    replace `y1' = `y1'  +  `uB_L4_`X''		if `ra'==0 & `task4'==1
                }	

                forvalues X = 1(1)$lambda5 {
                    generate double `uA_L5_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r_sim'))/(1-`r_sim')	if `ra'==0 & `task5'==1
                    generate double `uB_L5_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda5)^(1-`r_sim'))/(1-`r_sim')	if `ra'==0 & `task5'==1
                    replace `y0' = `y0'  +  `uA_L5_`X''		if `ra'==0 & `task5'==1
                    replace `y1' = `y1'  +  `uB_L5_`X''		if `ra'==0 & `task5'==1
                }	

                forvalues X = 1(1)$lambda6 {
                    generate double `uA_L6_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r_sim'))/(1-`r_sim')	if `ra'==0 & `task6'==1
                    generate double `uB_L6_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda6)^(1-`r_sim'))/(1-`r_sim')	if `ra'==0 & `task6'==1
                    replace `y0' = `y0'  +  `uA_L6_`X''		if `ra'==0 & `task6'==1
                    replace `y1' = `y1'  +  `uB_L6_`X''		if `ra'==0 & `task6'==1
                }	

                 generate double `uRatio' = (`y0'^(1/`muIDR'))/((`y0'^(1/`muIDR'))+(`y1'^(1/`muIDR')))

                 replace `sim_lnf' = ln(`uRatio')  				if  $ML_y1==0 & `ra'==0
                 replace `sim_lnf' = ln(1-`uRatio')  			if  $ML_y1==1 & `ra'==0
                 replace `sim_lnf' = (ln(1-`uRatio')/2)+(ln(`uRatio')/2)  	if  $ML_y1==-1 & `ra'==0

                 * accumulate the simulated likelihood and probability (`deltaM'>0 operative only if $truncated in effect)
                 if `deltaM'>0 {
                    replace `simmed_lnf' 	= `simmed_lnf' + exp(`sim_lnf')
                 }

                 drop `sim_lnf' `delta_sim' `uRatio' `y0' `y1'
                 drop `r_sim' `euB' `euA' `euRatio' `wp1' `wp2' `yA1' `yA2' `yB1' `yB2' 

                 forvalues X = 1(1)$lambda0 {
                    drop `uA_S_`X'' `uB_S_`X''
                 }	

                 forvalues X = 1(1)$lambda1 {
                    drop `uA_L1_`X'' `uB_L1_`X''
                 }	

                 forvalues X = 1(1)$lambda2 {
                    drop `uA_L2_`X'' `uB_L2_`X''
                 }	

                 forvalues X = 1(1)$lambda3 {
                    drop `uA_L3_`X'' `uB_L3_`X''
                 }	

                 forvalues X = 1(1)$lambda4 {
                    drop `uA_L4_`X'' `uB_L4_`X''
                 }	

                 forvalues X = 1(1)$lambda5 {
                    drop `uA_L5_`X'' `uB_L5_`X''
                 }	

                 forvalues X = 1(1)$lambda6 {
                    drop `uA_L6_`X'' `uB_L6_`X''
                 }
 
      }

      * get the average of the simulated draws
      replace `lnf' =  ln(`simmed_lnf'/`draws')

    }

end

* Define exponential model with CRRA, FL dual self specification (use delta here instead of LNdelta) -- log noise terms
program MLex_ra_crra_lam_r_

    args lnf r LNdelta LNmuRA LNmuIDR LNr_sd LNdelta_sd atrho12

	forvalues X = 1(1)$lambda0 {
	    tempvar uA_S_`X' uB_S_`X'
	}
	forvalues X = 1(1)$lambda1 {
	    tempvar uA_L1_`X' uB_L1_`X'
	}
	forvalues X = 1(1)$lambda2 {
	    tempvar uA_L2_`X' uB_L2_`X'
	}
	forvalues X = 1(1)$lambda3 {
	    tempvar uA_L3_`X' uB_L3_`X'
	}
	forvalues X = 1(1)$lambda4 {
	    tempvar uA_L4_`X' uB_L4_`X'
	}
	forvalues X = 1(1)$lambda5 {
	    tempvar uA_L5_`X' uB_L5_`X'
	}
	forvalues X = 1(1)$lambda6 {
	    tempvar uA_L6_`X' uB_L6_`X'
	}

    tempvar t tau mA mB y0 y1 uRatio delta delta_sd muRA muIDR

    tempvar ra r_sd prob1 prob2 mA1 mA2 mB1 mB2 euA euB euRatio cov12 yA1 yA2 yB1 yB2 wp1 wp2

    tempvar simmed_lnf sim_lnf delta_sim r_sim task1 task2 task3 task4 task5 task6 sigma_r sigma_d

	syntax varlist [if] [in] [fweight pweight] [, noLOg	Robust CLuster(varname)	svy	EForm *]
	marksample touse
	if "`svy'" != "" {
		svymarkout `touse'
	}
 
    quietly {

        * initialize horizon and payoffs
        generate double `t' 	= $ML_y2/360
        generate double `tau' 	= (30+$ML_y2)/360
        generate double `mA'	= $ML_y3
        generate double `mB'	= $ML_y4	
        generate double `prob1' = $ML_y5      
        generate double `prob2' = 1 - `prob1'
        generate double `ra' 	= $ML_y6
        generate double `mA1' 	= $ML_y7      
        generate double `mA2' 	= $ML_y8
        generate double `mB1' 	= $ML_y9
        generate double `mB2' 	= $ML_y10
        generate double `task1' = $ML_y11
        generate double `task2' = $ML_y12
        generate double `task3' = $ML_y13
        generate double `task4' = $ML_y14
        generate double `task5' = $ML_y15
        generate double `task6' = $ML_y16

        * transform parameters
        generate double `delta'     = exp(`LNdelta')
        generate double `muRA'      = exp(`LNmuRA')
        generate double `muIDR'     = exp(`LNmuIDR')
        scalar `r_sd'               = exp(`LNr_sd')
        scalar `delta_sd'           = exp(`LNdelta_sd')

        * Generate parameters needed for the random effects
        scalar `sigma_r'	=(exp(`LNr_sd'))^2
        scalar `sigma_d'	=(exp(`LNdelta_sd'))^2
        scalar `cov12'		=[exp(2*`atrho12')-1]/[exp(2*`atrho12')+1]* (exp(`LNr_sd'))*(exp(`LNdelta_sd'))

        * define covariance matrix
    	matrix p = (`sigma_r', `cov12' \ `cov12', `sigma_d')
        $noise matrix list p
	
        * get the Cholesky factors calculated
        capture matrix c = cholesky(p)
       	if _rc ~= 0 {
            $noise display "Warning -- matrix probably not positive definite: using last matrix" 
        }
        else {
            $noise matrix list p
            $noise matrix list c
        }
        local c11 = c[1,1]
        local c21 = c[2,1]
        local c22 = c[2,2]
    	$noise display "Cholesky factors: c11 is `c11' c21 is `c21' and c22 is `c22'"

       	* loop over likelihood evaluator with $draw random draws and accumulate simulated likelihoods
        generate double `simmed_lnf' = 0
        local draws = 0

        forvalues d=1/$draw {
	
                * generate bivariate normal or lognormal deviates
                if "$lognormal" == "n" {
                    generate double `r_sim'		= `r'		+ `c11'*invnormal(h1_`d')
                    generate double `delta_sim'		= `delta'	+ `c21'*invnormal(h1_`d') + `c22'*invnormal(h2_`d')
                }
                else {
                    generate double `r_sim'		= exp(`r'		                  + `c11'invnormal(h1_`d'))
                    generate double `delta_sim'		= exp(`delta'	+ `c21'*invnormal(h1_`d') + `c22'*invnormal(h2_`d'))
                }

                * truncate if requested
                if "$truncate" == "y" {
                    summ `delta_sim'
                    local deltaM = r(mean)
                    if `deltaM'>0 {
                        local draws = `draws'+1
                    }
                }
                else {
                    local deltaM = 1
                    local draws = $draw
                }

                * estimating over risk responses
                    generate double `yA1' = (($omega+`mA1')^(1-`r_sim'))/(1-`r_sim')		if `ra'==1
                    generate double `yA2' = (($omega+`mA2')^(1-`r_sim'))/(1-`r_sim')		if `ra'==1
                    generate double `yB1' = (($omega+`mB1')^(1-`r_sim'))/(1-`r_sim')		if `ra'==1
                    generate double `yB2' = (($omega+`mB2')^(1-`r_sim'))/(1-`r_sim')		if `ra'==1

                generate double `wp1' = `prob1'					if `ra'==1	
                generate double `wp2' = `prob2'					if `ra'==1
                generate double `euA' = (`wp1'*`yA1')+(`wp2'*`yA2')		if `ra'==1      
                generate double `euB' = (`wp1'*`yB1')+(`wp2'*`yB2')		if `ra'==1

                generate double `euRatio' = (`euA'^(1/`muRA'))/((`euA'^(1/`muRA'))+(`euB'^(1/`muRA')))	if `ra'==1
 
            	generate double `sim_lnf' = 0
            	replace `sim_lnf' = ln(`euRatio')   				if $ML_y1==0		& `ra'==1
            	replace `sim_lnf' = ln(1-`euRatio')                                	if $ML_y1==1		& `ra'==1
            	replace `sim_lnf' = (ln(1-`euRatio')/2)+(ln(`euRatio')/2)		if $ML_y1==-1		& `ra'==1

                * estimating over time responses
                generate double `y0' = 0
                generate double `y1' = 0

                * estimating over time responses
                forvalues X = 1(1)$lambda0 {
                    generate double `uA_S_`X'' = (1 / ((1+`delta_sim')^(30/360 + (`X'-1)/360 )) ) * (($omega + `mA'/$lambda0)^(1-`r_sim'))/(1-`r_sim')	if `ra'==0
                    generate double `uB_S_`X'' = (1 / ((1+`delta_sim')^(30/360 + (`X'-1)/360 )) ) * (($omega                )^(1-`r_sim'))/(1-`r_sim')	if `ra'==0
                    replace `y0' = `y0' +  `uA_S_`X''		if `ra'==0
                    replace `y1' = `y1' +  `uB_S_`X''		if `ra'==0
                }	

                forvalues X = 1(1)$lambda1 {
                    generate double `uA_L1_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r_sim'))/(1-`r_sim')	if `ra'==0 & `task1'==1
                    generate double `uB_L1_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda1)^(1-`r_sim'))/(1-`r_sim')	if `ra'==0 & `task1'==1
                    replace `y0' = `y0'  +  `uA_L1_`X''		if `ra'==0 & `task1'==1
                    replace `y1' = `y1'  +  `uB_L1_`X''		if `ra'==0 & `task1'==1
                }	

                forvalues X = 1(1)$lambda2 {
                    generate double `uA_L2_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r_sim'))/(1-`r_sim')	if `ra'==0 & `task2'==1
                    generate double `uB_L2_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda2)^(1-`r_sim'))/(1-`r_sim')	if `ra'==0 & `task2'==1
                    replace `y0' = `y0'  +  `uA_L2_`X''		if `ra'==0 & `task2'==1
                    replace `y1' = `y1'  +  `uB_L2_`X''		if `ra'==0 & `task2'==1
                }	

                forvalues X = 1(1)$lambda3 {
                    generate double `uA_L3_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r_sim'))/(1-`r_sim')	if `ra'==0 & `task3'==1
                    generate double `uB_L3_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda3)^(1-`r_sim'))/(1-`r_sim')	if `ra'==0 & `task3'==1
                    replace `y0' = `y0'  +  `uA_L3_`X''		if `ra'==0 & `task3'==1
                    replace `y1' = `y1'  +  `uB_L3_`X''		if `ra'==0 & `task3'==1
                }	

                forvalues X = 1(1)$lambda4 {
                    generate double `uA_L4_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r_sim'))/(1-`r_sim')	if `ra'==0 & `task4'==1
                    generate double `uB_L4_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda4)^(1-`r_sim'))/(1-`r_sim')	if `ra'==0 & `task4'==1
                    replace `y0' = `y0'  +  `uA_L4_`X''		if `ra'==0 & `task4'==1
                    replace `y1' = `y1'  +  `uB_L4_`X''		if `ra'==0 & `task4'==1
                }	

                forvalues X = 1(1)$lambda5 {
                    generate double `uA_L5_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r_sim'))/(1-`r_sim')	if `ra'==0 & `task5'==1
                    generate double `uB_L5_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda5)^(1-`r_sim'))/(1-`r_sim')	if `ra'==0 & `task5'==1
                    replace `y0' = `y0'  +  `uA_L5_`X''		if `ra'==0 & `task5'==1
                    replace `y1' = `y1'  +  `uB_L5_`X''		if `ra'==0 & `task5'==1
                }	

                forvalues X = 1(1)$lambda6 {
                    generate double `uA_L6_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (($omega                )^(1-`r_sim'))/(1-`r_sim')	if `ra'==0 & `task6'==1
                    generate double `uB_L6_`X'' = (1 / ((1+`delta_sim')^(`tau' + (`X'-1)/360 )) ) * (($omega + `mB'/$lambda6)^(1-`r_sim'))/(1-`r_sim')	if `ra'==0 & `task6'==1
                    replace `y0' = `y0'  +  `uA_L6_`X''		if `ra'==0 & `task6'==1
                    replace `y1' = `y1'  +  `uB_L6_`X''		if `ra'==0 & `task6'==1
                }	

                 generate double `uRatio' = (`y0'^(1/`muIDR'))/((`y0'^(1/`muIDR'))+(`y1'^(1/`muIDR')))

                 replace `sim_lnf' = ln(`uRatio')  				if  $ML_y1==0 & `ra'==0
                 replace `sim_lnf' = ln(1-`uRatio')  			if  $ML_y1==1 & `ra'==0
                 replace `sim_lnf' = (ln(1-`uRatio')/2)+(ln(`uRatio')/2)  	if  $ML_y1==-1 & `ra'==0

                 * accumulate the simulated likelihood and probability (`deltaM'>0 operative only if $truncated in effect)
                 if `deltaM'>0 {
                    replace `simmed_lnf' 	= `simmed_lnf' + exp(`sim_lnf')
                 }

                 drop `sim_lnf' `delta_sim' `uRatio' `y0' `y1'
                 drop `r_sim' `euB' `euA' `euRatio' `wp1' `wp2' `yA1' `yA2' `yB1' `yB2' 

                 forvalues X = 1(1)$lambda0 {
                    drop `uA_S_`X'' `uB_S_`X''
                 }	

                 forvalues X = 1(1)$lambda1 {
                    drop `uA_L1_`X'' `uB_L1_`X''
                 }	

                 forvalues X = 1(1)$lambda2 {
                    drop `uA_L2_`X'' `uB_L2_`X''
                 }	

                 forvalues X = 1(1)$lambda3 {
                    drop `uA_L3_`X'' `uB_L3_`X''
                 }	

                 forvalues X = 1(1)$lambda4 {
                    drop `uA_L4_`X'' `uB_L4_`X''
                 }	

                 forvalues X = 1(1)$lambda5 {
                    drop `uA_L5_`X'' `uB_L5_`X''
                 }	

                 forvalues X = 1(1)$lambda6 {
                    drop `uA_L6_`X'' `uB_L6_`X''
                 }
 
      }

      * get the average of the simulated draws
      replace `lnf' =  ln(`simmed_lnf'/`draws')

    }

end

