/* Setup.do sets up the data for analysis */

* get data
use Data, replace

* define the complex survey parameters
svyset [pweight=weight], strata(county) psu(id)

* identify the individual characteristics
global demog "female young middle old single kids owner retired student skilled longedu IncLow IncHigh copen city"

* identify treatments
global skew "skewLO skewHI"

* identify the risk aversion tasks
global Rtasks "RTask2 RTask3 RTask4"

* identify the discount rate tasks
global DRtasks "DRTask2 DRTask3 DRTask4 DRTask5 DRTask6"

* summarize right hand side variables
summ $demog

* tabulate the number of indifference choices
tab choice ra, missing col

* Set values for the 95% graphs
* evaluate the parameters for different horizons levels -- and predicting the confidence intervals (times factor of $HorizonSCALE)
global HorizonLO  "1"
global HorizonInc "2"
global HorizonHI  "29"
global HorizonVIEW "0"
global HorizonSCALE "10"
global yfig "0(0.05)0.15"

* specify choice variables
generate choices =.
replace  choices = 0 if choice=="a"
replace  choices = 1 if choice=="b"
replace  choices =-1 if choice=="i"

* default algorithm
global alg "dfp"

* increments for RRA evaluations
global yLO   "0"
global yINC  "100"
global yHI   "5000"
global yVIEW "0"

* specify "noi" instead of "qui" if you want debugging displays
global noise "qui"

* time horizon in days
global days=7
global lambda=1/$days

* daily background consumption level
global omega = 1000

* daily background consumption level in risk neutral models
global bc = 0

* constraints for Prelec-hyperbolic model
global alphaDLO=0
global alphaDHI=2

* use lognormal distribution of random effects
global lognormal "n"

* save for restarts
save tmp, replace

* clean up
erase Data.dta
